@@ -18,7 +18,7 @@ echo ""
 ./abi-extract.pl -N it-IT      
 ./abi-extract.pl -N pt-BR 
 ./abi-extract.pl -N zh-TW 
-./abi-extract.pl -N cy-CY 
+./abi-extract.pl -N cy-GB 
 ./abi-extract.pl -N fi-FI 
 ./abi-extract.pl -N lt-LT 
 ./abi-extract.pl -N pt-PT 
