@@ -18,6 +18,12 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
  * 02111-1307, USA.
  */
+ 
+/*
+ * Port to Maemo Development Platform
+ * Author: INdT - Renato Araujo <renato.filho@indt.org.br>
+ */
+
 
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
@@ -46,6 +52,10 @@
 #include "ut_string_class.h"
 #include "xap_UnixDialogHelper.h"
 #include "ap_Menu_Id.h"
+#ifdef HAVE_HILDON
+#include <hildon-widgets/hildon-appview.h>
+#endif
+
 
 // set up these replacement icons
 #include "stock/menu_about.xpm"
@@ -431,22 +441,20 @@ bool EV_UnixMenu::menuEvent(XAP_Menu_Id id)
 	return true;
 }
 
-static char _ev_get_underlined_char(const char * szString)
+static guint _ev_get_underlined_char(const char * szString)
 {
 
 	UT_ASSERT(szString);
 	
-	// return the char right after the underline
-	const char * p = szString;
-	while (*p && *(p+1))
+	// return the keycode right after the underline
+	const UT_UCS4String str(szString);
+	for (UT_uint32 i = 0; i + 1 < str.length(); )
 	{
-		if (*p == '_')
-			return *++p;
-		else
-			p++;
+		if (str[i++] == '_')
+			return gdk_unicode_to_keyval(str[i]);
 	}
 
-	return 0;
+	return GDK_VoidSymbol;
 }
 
 static void _ev_strip_underline(char * bufResult,
@@ -523,16 +531,6 @@ const char * EV_UnixMenu::s_getStockPixmapFromId (int id)
 		{AP_MENU_ID_FILE_EXIT, GTK_STOCK_QUIT},
 		{AP_MENU_ID_FILE_REVERT, GTK_STOCK_REVERT_TO_SAVED},
 
-		{AP_MENU_ID_FILE_RECENT_1, GTK_STOCK_NEW},
-		{AP_MENU_ID_FILE_RECENT_2, GTK_STOCK_NEW},
-		{AP_MENU_ID_FILE_RECENT_3, GTK_STOCK_NEW},
-		{AP_MENU_ID_FILE_RECENT_4, GTK_STOCK_NEW},
-		{AP_MENU_ID_FILE_RECENT_5, GTK_STOCK_NEW},
-		{AP_MENU_ID_FILE_RECENT_6, GTK_STOCK_NEW},
-		{AP_MENU_ID_FILE_RECENT_7, GTK_STOCK_NEW},
-		{AP_MENU_ID_FILE_RECENT_8, GTK_STOCK_NEW},
-		{AP_MENU_ID_FILE_RECENT_9, GTK_STOCK_NEW},		
-
 		{AP_MENU_ID_EDIT_UNDO, GTK_STOCK_UNDO},
 		{AP_MENU_ID_EDIT_REDO, GTK_STOCK_REDO},
 		{AP_MENU_ID_EDIT_CUT, GTK_STOCK_CUT},
@@ -619,7 +617,19 @@ const char * EV_UnixMenu::s_getStockPixmapFromId (int id)
 		{ AP_MENU_ID_TABLE_SELECT_ROW, "Menu_AbiWord_Select_Row"},
 		{ AP_MENU_ID_TABLE_SELECT_TABLE, "Menu_AbiWord_Select_Table"},
 		{ AP_MENU_ID_INSERT_DATETIME, "Menu_AbiWord_Insert_Date"},
-		
+
+		// textbox
+		{ AP_MENU_ID_EDIT_CUT_FRAME, GTK_STOCK_CUT},
+		{ AP_MENU_ID_EDIT_COPY_FRAME, GTK_STOCK_COPY},
+		{ AP_MENU_ID_EDIT_SELECT_FRAME, "Menu_AbiWord_Select_All"},
+		{ AP_MENU_ID_EDIT_DELETEFRAME, GTK_STOCK_DELETE},
+
+		// "wrapped" image
+		{ AP_MENU_ID_FILE_SAVEIMAGE, GTK_STOCK_SAVE_AS}, 
+		{ AP_MENU_ID_EDIT_CUTIMAGE, GTK_STOCK_CUT}, 
+		{ AP_MENU_ID_EDIT_COPYIMAGE, GTK_STOCK_COPY}, 
+		{ AP_MENU_ID_EDIT_DELETEIMAGE, GTK_STOCK_DELETE}, 
+
 		{AP_MENU_ID__BOGUS2__, NULL}
 	};
 	
@@ -736,8 +746,13 @@ bool EV_UnixMenu::synthesizeMenu(GtkWidget * wMenuRoot)
 				// but instead make a label with no underlines (and no accelerators).
 				
 				// get the underlined value from the candidate label
-				guint keyCode = _ev_get_underlined_char(buf);
-
+				guint keyCode;
+#ifdef HAVE_HILDON
+				keyCode = GDK_VoidSymbol;
+#else
+				keyCode = _ev_get_underlined_char(buf);
+#endif
+				
 				// GTK triggers the menu accelerators off of MOD1 ***without
 				// regard to what XK_ keysym is bound to it.  therefore, if
 				// MOD1 is bound to XK_Alt_{L,R}, we do the following.
@@ -749,12 +764,12 @@ bool EV_UnixMenu::synthesizeMenu(GtkWidget * wMenuRoot)
 				// version of the underlined char, since all the menus ignore upper
 				// case (SHIFT-MOD1-[char]) invokations of accelerators.
 
-				if (bAltOnMod1)
+				if (keyCode != GDK_VoidSymbol && bAltOnMod1)
 				{
 					EV_EditEventMapper * pEEM = XAP_App::getApp()->getEditEventMapper();
 					UT_ASSERT(pEEM);
 					EV_EditMethod * pEM = NULL;
-					pEEM->Keystroke(EV_EKP_PRESS|EV_EMS_ALT|tolower(keyCode),&pEM);
+					pEEM->Keystroke(EV_EKP_PRESS|EV_EMS_ALT|keyCode,&pEM);
 
 					// if the pointer is valid, there is a conflict
 					bConflict = (pEM != NULL);
@@ -868,6 +883,9 @@ bool EV_UnixMenu::synthesizeMenu(GtkWidget * wMenuRoot)
 		}
 	}
 
+#ifdef HAVE_HILDON	 /* don't use accelerators in Hildon -- not enough screen space */
+#else
+
 	// make sure our last item on the stack is the one we started with
 	GtkWidget * wDbg = NULL;
 	bResult = stack.pop(reinterpret_cast<void **>(&wDbg));
@@ -880,6 +898,8 @@ bool EV_UnixMenu::synthesizeMenu(GtkWidget * wMenuRoot)
 	gtk_window_add_accel_group(GTK_WINDOW(static_cast<XAP_UnixFrameImpl *>(m_pFrame->getFrameImpl())->getTopLevelWindow()), m_accelGroup);
 	gtk_accel_group_lock(m_accelGroup);
 	
+#endif
+
 	return true;
 }
 
@@ -1163,14 +1183,23 @@ void  EV_UnixMenuBar::destroy(void)
 
 bool EV_UnixMenuBar::synthesizeMenuBar()
 {
-	GtkWidget * wVBox = static_cast<XAP_UnixFrameImpl *>(m_pFrame->getFrameImpl())->getVBoxWidget();
 
 	// Just create, don't show the menu bar yet.  It is later added and shown
+#ifdef HAVE_HILDON /* in hildon sdk you have get menu_bar from mainWidonw */
+	GtkWidget * wWidget = static_cast<XAP_UnixFrameImpl *>(m_pFrame->getFrameImpl())->getTopLevelWindow();
+	m_wMenuBar = GTK_WIDGET(hildon_appview_get_menu(HILDON_APPVIEW(wWidget)));
+#else
+	GtkWidget * wVBox = static_cast<XAP_UnixFrameImpl *>(m_pFrame->getFrameImpl())->getVBoxWidget();
 	m_wMenuBar = gtk_menu_bar_new();
-	synthesizeMenu(m_wMenuBar);
-	gtk_widget_show(m_wMenuBar);
+#endif
 
+	synthesizeMenu(m_wMenuBar);
+	gtk_widget_show_all(m_wMenuBar);
+
+#ifdef HAVE_HILDON	 /* in hildon no need */
+#else
 	gtk_box_pack_start(GTK_BOX(wVBox), m_wMenuBar, FALSE, TRUE, 0);
+#endif	
 
 	return true;
 }
@@ -1289,6 +1318,8 @@ GtkWidget * EV_UnixMenu::s_createNormalMenuEntry(int id, const bool isCheckable,
 			w = gtk_radio_menu_item_new_with_mnemonic (NULL, buf);
 		}
 	
+#ifdef HAVE_HILDON /* not necessary in hildon */		
+#else
 	if (szMnemonicName && *szMnemonicName)
 	  {
 		  guint accelKey = 0;
@@ -1298,6 +1329,7 @@ GtkWidget * EV_UnixMenu::s_createNormalMenuEntry(int id, const bool isCheckable,
 		  // are handled at a lower level (we just get an accel label)
 		  gtk_widget_add_accelerator (w, "activate", m_accelGroup, accelKey, acMods, GTK_ACCEL_VISIBLE);
 	  }
+#endif	  
 	
 	gtk_widget_show(w);
 	
