@@ -36,6 +36,7 @@
 #include "xap_Frame.h"
 #include "xap_Dialog_Id.h"
 #include "xap_Win32Dlg_Print.h"
+#include "xap_EncodingManager.h"
 
 #include "ut_debugmsg.h"
 #include "ut_assert.h"
@@ -92,6 +93,8 @@ void GR_Win32Graphics::_constructorCommonCode(HDC hdc)
 {
 	UT_ASSERT(hdc);
 
+	m_iDCFontAllocNo = 0;
+	m_iPrintDCFontAllocNo = 0;
 	m_hdc = hdc;
 	m_printHDC = NULL;
 	m_nPrintLogPixelsY = 0;
@@ -330,7 +333,18 @@ GR_Font* GR_Win32Graphics::_findFont(const char* pszFontFamily,
 	else if (0 == UT_stricmp(pszFontFamily, "monospace"))
 		lf.lfPitchAndFamily = DEFAULT_PITCH | FF_MODERN;
 	else
-		strcpy(lf.lfFaceName, pszFontFamily);
+	{
+		// NB: lf.lfFaceName is a statically allocated char buffer of len 
+		// LF_FACESIZE. strToNative() will truncate the string at 500 bytes,
+		// but as LF_FACESIZE is generally 32, this shouldn't matter.
+		//
+		// This convertion is needed because the windows API expects fontnames
+		// in the native encoding and later lf will be passing to a windows
+		// API funcion (CreateFontIndirect()).
+		strncpy(lf.lfFaceName, 
+		        getApp()->getEncodingManager()->strToNative(pszFontFamily, "UTF-8", false),
+		        LF_FACESIZE);
+	}
 
 	// Get character set value from the font itself
 	LOGFONT enumlf = { 0 };
@@ -463,6 +477,7 @@ void GR_Win32Graphics::drawChars(const UT_UCSChar* pChars,
 	int iLength = iLengthOrig;
 	HFONT hFont = m_pFont->getDisplayFont(this);
 	SelectObject(m_hdc, hFont);
+	m_iDCFontAllocNo = m_pFont->getAllocNumber();
 	SetTextAlign(m_hdc, TA_LEFT | TA_TOP);
 	SetBkMode(m_hdc, TRANSPARENT);		// TODO: remember and reset?
 
@@ -638,7 +653,7 @@ UT_uint32 GR_Win32Graphics::getFontHeight()
 {
 	UT_return_val_if_fail( m_pFont, 0 );
 
-	return (UT_uint32)((m_pFont->getHeight(m_hdc, m_printHDC)) * getResolution() / getDeviceResolution());
+	return (UT_uint32)(m_pFont->getHeight(m_hdc, m_printHDC));
 }
 
 UT_uint32 GR_Win32Graphics::getFontAscent(GR_Font* fnt)
@@ -653,7 +668,7 @@ UT_uint32 GR_Win32Graphics::getFontAscent(GR_Font* fnt)
 UT_uint32 GR_Win32Graphics::getFontAscent()
 {
 	UT_return_val_if_fail( m_pFont, 0 );
-	return (UT_uint32)((m_pFont->getAscent(m_hdc, m_printHDC)) * getResolution() / getDeviceResolution());
+	return (UT_uint32)(m_pFont->getAscent(m_hdc, m_printHDC));
 }
 
 UT_uint32 GR_Win32Graphics::getFontDescent(GR_Font* fnt)
@@ -668,7 +683,7 @@ UT_uint32 GR_Win32Graphics::getFontDescent(GR_Font* fnt)
 UT_uint32 GR_Win32Graphics::getFontDescent()
 {
 	UT_return_val_if_fail( m_pFont, 0 );
-	return (UT_uint32)((m_pFont->getDescent(m_hdc, m_printHDC)) * getResolution() / getDeviceResolution());
+	return (UT_uint32)(m_pFont->getDescent(m_hdc, m_printHDC));
 }
 
 void GR_Win32Graphics::getCoverage(UT_NumberVector& coverage)
@@ -919,8 +934,10 @@ bool GR_Win32Graphics::startPrint(void)
 bool GR_Win32Graphics::startPage(const char * szPageLabel, UT_uint32 pageNumber,
 									bool bPortrait, UT_uint32 iWidth, UT_uint32 iHeight)
 {
-	iWidth = (UT_sint32)((double)_tduR(iWidth) * m_fXYRatio);
-	iHeight = _tduR(iHeight);
+	// need these in 10th of milimiters
+	iWidth = (int)((double)iWidth * m_fXYRatio * 254.0 / (double)getResolution() + 0.5);
+	iHeight = iHeight * 254 / getResolution();
+
 	if (m_bStartPage)
 	{
 		EndPage(m_hdc);
@@ -930,8 +947,12 @@ bool GR_Win32Graphics::startPage(const char * szPageLabel, UT_uint32 pageNumber,
 	if (m_hDevMode)
 	{
 		DEVMODE *pDevMode = (DEVMODE*) GlobalLock(m_hDevMode);
-		pDevMode->dmFields = DM_ORIENTATION;
+		pDevMode->dmFields = DM_ORIENTATION | DM_PAPERLENGTH | DM_PAPERWIDTH;
 		pDevMode->dmOrientation = (bPortrait) ? DMORIENT_PORTRAIT : DMORIENT_LANDSCAPE;
+		pDevMode->dmPaperSize = 0;
+		pDevMode->dmPaperLength = iHeight;
+		pDevMode->dmPaperWidth = iWidth;
+		
 		GlobalUnlock(m_hDevMode);
 
 		// call DocumentProperties() on the DEVMODE to ensure changes propagate down into
@@ -1149,13 +1170,40 @@ void GR_Win32Graphics::drawImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDes
 	UT_uint32 iSizeOfColorData = pDIB->bmiHeader.biClrUsed * sizeof(RGBQUAD);
 	UT_Byte* pBits = ((unsigned char*) pDIB) + pDIB->bmiHeader.biSize + iSizeOfColorData;
 
-	int iRes = StretchDIBits(m_hdc,
+	int iRes = 0;
+
+#if 0
+	// AlphaBlend is supported from win98 upwards
+	// we need to turn the DIB into DC first
+	HDC memDC = CreateCompatibleDC(m_hdc);
+
+	if(memDC)
+	{
+		// need to convert the dib pointer to handle here ...
+
+		// now select into the memory dc and call AlphaBlend
+		SelectObject(memDC, hDIB);
+		iRes = AlphaBlend(m_hdc,
+						  xDest, yDest,
+						  pImg->getDisplayWidth(), pImg->getDisplayHeight(),
+						  memDC,
+						  0, 0,
+						  pDIB->bmiHeader.biWidth, pDIB->bmiHeader.biHeight,
+						  blendfnct);
+	}
+	
+#endif
+	
+	if(!iRes)
+	{
+		iRes = StretchDIBits(m_hdc,
 							 xDest, yDest,
 							 pImg->getDisplayWidth(), pImg->getDisplayHeight(),
 							 0, 0,
 							 pDIB->bmiHeader.biWidth, pDIB->bmiHeader.biHeight,
 							 pBits, pDIB, DIB_RGB_COLORS, SRCCOPY);
-
+	}
+	
 	if (iRes == GDI_ERROR)
 	{
 		DWORD err = GetLastError();
@@ -1382,7 +1430,11 @@ void GR_Font::s_getGenericFontProperties(const char * szFontName,
 	else if (UT_stricmp(szFontName, "monospace") == 0)
 		lf.lfPitchAndFamily = DEFAULT_PITCH | FF_MODERN;
 	else
-		strcpy(lf.lfFaceName, szFontName);
+	{
+			strncpy(lf.lfFaceName, 
+				    XAP_EncodingManager::get_instance()->strToNative(szFontName, "UTF-8", false),
+			        LF_FACESIZE);
+	}
 
 	// let the system create the font with the given name or
 	// properties and then query it and see what was actually
@@ -1450,10 +1502,12 @@ GR_Win32Font::GR_Win32Font(LOGFONT & lf, double fPoints, HDC hdc, HDC printHDC)
 	
 	if(hdc != printHDC)
 	{
-		int nLogPixelsY      = GetDeviceCaps(hdc, LOGPIXELSY);
 		int nPrintLogPixelsY = GetDeviceCaps(printHDC, LOGPIXELSY);
 
-		lf.lfHeight = MulDiv(lf.lfHeight, nPrintLogPixelsY, nLogPixelsY);
+		// use the point size rather than the screen pixel size to minimise rounding error
+		// (lfHeight already carries a rounding error from conversion pts -> screen pixels)
+		// lf.lfHeight = MulDiv(lf.lfHeight, nPrintLogPixelsY, nLogPixelsY);
+		lf.lfHeight =  (int)(-fPoints * (double)nPrintLogPixelsY / 72.00 + 0.5);
 		printFont = CreateFontIndirect(&lf);
 		
 		if(!printFont)
@@ -1496,7 +1550,7 @@ GR_Win32Font::GR_Win32Font(LOGFONT & lf, double fPoints, HDC hdc, HDC printHDC)
 			GetTextFace(printHDC, 1000, lpFaceName );
 
 			_updateFontYMetrics(hdc, printHDC);
-			
+
 			UT_String_sprintf(m_hashKey, "%s-%d-%d-%d-%d-%d",
 							  lpFaceName,
 							  m_tm.tmHeight, m_tm.tmWeight, m_tm.tmItalic, m_tm.tmUnderlined,
@@ -1601,13 +1655,18 @@ void GR_Win32Font::_updateFontYMetrics(HDC hdc, HDC printHDC)
 	{
 		GetTextMetrics(printHDC,&m_tm);
 
-		// now we have to scale the metrics down from the printHDC to primary dc
+		// now we have to scale the metrics down from the printHDC to our layout units
+		// 
+		// (we used to scale the metrics down to the device resolution, but that since we
+		// then scale it back up to layout units all the time, this caused us huge
+		// rounding error) Tomas, 16/9/05
+		// 
 		// NB: we do not want to obtain the metrics directly for the primary dc
 		// Windows is designed to get slightly bigger font for the screen than the
 		// user asks for (MS say to improve readibility), and this causes us
 		// non-WYSIWYG behaviour (basically our characters are wider on screen than on
 		// the printer and our lines are further apart)
-		int nLogPixelsY      = GetDeviceCaps(hdc, LOGPIXELSY);
+		int nLogPixelsY      = GR_Graphics::getResolution();
 		int nPrintLogPixelsY = GetDeviceCaps(printHDC, LOGPIXELSY);
 
 		if(nLogPixelsY != nPrintLogPixelsY)
@@ -1698,17 +1757,18 @@ bool GR_Win32Font::glyphBox(UT_UCS4Char g, UT_Rect & rec, GR_Graphics * pG)
 	GLYPHMETRICS gm;
 	MAT2 m = {{0,1}, {0,0}, {0,0}, {0,1}};
 
-	int nLogPixelsY      = GetDeviceCaps(hdc, LOGPIXELSY);    // resolution of primary device
 	int nPrintLogPixelsY = GetDeviceCaps(printDC, LOGPIXELSY);// resolution of printer
-
-	int nLogPixelsX      = GetDeviceCaps(hdc, LOGPIXELSX);
 	int nPrintLogPixelsX = GetDeviceCaps(printDC, LOGPIXELSX);
 
 	UT_uint32 pixels = getUnscaledHeight();
 
 	// scale the pixel size to the printer resolution and get the font
 	if(!m_bGUIFont)
-		pixels = MulDiv(pixels, nPrintLogPixelsY, 72);
+	{
+		// use point size to reduce rounding errors
+		// pixels = MulDiv(pixels, nPrintLogPixelsY, 72);
+		pixels = (int)(m_fPointSize * (double)nPrintLogPixelsY / 72.0 + 0.5);
+	}
 	
 	HFONT pFont = getFontFromCache(pixels, false, 100);
 	if (!pFont)
@@ -1720,7 +1780,14 @@ bool GR_Win32Font::glyphBox(UT_UCS4Char g, UT_Rect & rec, GR_Graphics * pG)
 	// select our font into the printer DC
 	HGDIOBJ hRet = SelectObject(printDC, pFont);
 	UT_return_val_if_fail( hRet != (void*)GDI_ERROR, false);
-		
+
+	// remember which font we selected into the dc
+	if(printDC == pWin32Gr->getPrintDC())
+		pWin32Gr->setPrintDCFontAllocNo(getAllocNumber());
+
+	if(printDC == pWin32Gr->getPrimaryDC())
+		pWin32Gr->setDCFontAllocNo(getAllocNumber());
+	
 	if (printDC != m_hdc)
 	{
 		// invalidate cached info when we change hdc's.
@@ -1738,8 +1805,36 @@ bool GR_Win32Font::glyphBox(UT_UCS4Char g, UT_Rect & rec, GR_Graphics * pG)
 	}
 	else
 	{
-		// TODO: an UT_UCS4Char -> ANSI conversion here, then call GetGlyphOutlineA() ...
-		UT_return_val_if_fail( UT_NOT_IMPLEMENTED, false );
+		// GetGlyphOutlineA() ... using the GGO_GLYPH_INDEX = 0x0080 to get the ttf glyph 
+		// for the unicode character set in ANSI
+
+		// first of all, we need to translate the ucs4 value to the index; we use
+		// GetCharacterPlacementW() for this. Event though the docs state that this
+		// function is only available on win9x via unicows, this is in fact incorrect
+		// (whether the 9x version of the function is fully functional I do not know, but
+		// it does glyph index lookup).
+		UINT iIndx;
+		GCP_RESULTSW gcpResult = {0};
+		gcpResult.lStructSize = sizeof(GCP_RESULTS);
+		
+		// w32api changed lpGlyphs from UINT * to LPWSTR to match MS PSDK in w32api v2.4
+#if defined(__MINGW32__) && (__W32API_MAJOR_VERSION == 2 && __W32API_MINOR_VERSION < 4)
+		gcpResult.lpGlyphs = (UINT *) &iIndx;	// Character glyphs
+#else			
+		gcpResult.lpGlyphs = (LPWSTR) &iIndx;	// Character glyphs
+#endif			
+		gcpResult.nGlyphs = 1;  // Array size
+
+		if(GetCharacterPlacementW(printDC, (LPCWSTR)&g, 1, 0, &gcpResult, 0))
+		{
+			iRet = GetGlyphOutlineA(printDC, iIndx, 0x0080 | GGO_METRICS, &gm, 0, NULL, &m);
+		}
+		else
+		{
+			// OK, the GetCharacterPlacementW function probably not present, we just do
+			// our best with the ANSI stuff
+			iRet = GetGlyphOutlineA(printDC, g, GGO_METRICS, &gm, 0, NULL, &m);
+		}
 	}
 	
 	if(GDI_ERROR == iRet)
@@ -1750,19 +1845,13 @@ bool GR_Win32Font::glyphBox(UT_UCS4Char g, UT_Rect & rec, GR_Graphics * pG)
 	rec.width  = gm.gmBlackBoxX;
 	rec.height = gm.gmBlackBoxY;
 	
-	// if the primary DC is not the printer, we now have to scale the metrics to the
-	// screen resolution
-	if(nLogPixelsY != nPrintLogPixelsY)
-	{
-		rec.height  = MulDiv(rec.height, nLogPixelsY, nPrintLogPixelsY);
-		rec.top  = MulDiv(rec.top, nLogPixelsY, nPrintLogPixelsY);
-	}
+	// the metrics are in device units, scale them to layout
+	int iResolution = pG->getResolution();
+	rec.height  = MulDiv(rec.height, iResolution, nPrintLogPixelsY);
+	rec.top  = MulDiv(rec.top, iResolution, nPrintLogPixelsY);
 
-	if(nLogPixelsX != nPrintLogPixelsX)
-	{
-		rec.width  = MulDiv(rec.width, nLogPixelsX, nPrintLogPixelsX);
-		rec.left  = MulDiv(rec.left, nLogPixelsX, nPrintLogPixelsX);
-	}
+	rec.width  = MulDiv(rec.width, iResolution, nPrintLogPixelsX);
+	rec.left  = MulDiv(rec.left, iResolution, nPrintLogPixelsX);
 
 	UT_DEBUGMSG(("gr_Win32Graphics::glyphBox(l=%d, t=%d, w=%d, h=%d\n", rec.left, rec.top, rec.width, rec.height));
 	return true;
