@@ -112,7 +112,7 @@ class ABI_EXPORT GR_Font
 // rec.top = distance from the origin to the top of the glyph
 // rec.height = total height of the glyph
 
-	virtual bool glyphBox(UT_UCS4Char g, UT_Rect & rec) const = 0;
+	virtual bool glyphBox(UT_UCS4Char g, UT_Rect & rec, GR_Graphics * pG) = 0;
 	static  bool s_doesGlyphExist(UT_UCS4Char g, void *instance)
 	{
 		UT_return_val_if_fail(instance, false);
@@ -714,6 +714,14 @@ class ABI_EXPORT GR_Graphics
 						   UT_sint32 xoff,
 						   UT_sint32 yoff,
 						   int* pCharWidths = NULL) = 0;
+
+	virtual void drawCharsRelativeToBaseline(const UT_UCSChar* pChars,
+											 int iCharOffset,
+											 int iLength,
+											 UT_sint32 xoff,
+											 UT_sint32 yoff,
+											 int* pCharWidths = NULL);
+	
 	virtual GR_Image *	  genImageFromRectangle(const UT_Rect & r) = 0;
 
 	XAP_App * getApp() const {return m_pApp;}
