@@ -34,6 +34,10 @@ UT_BEGIN_EXTERN_C
 /* Units of UT_LAYOUT_RESOLUTION = dots per inch */
 #define UT_LAYOUT_RESOLUTION				1440
 
+/* 2^31 -1 */
+
+#define UT_INT_MAX 2147483647
+
 typedef enum _ut_dimension { DIM_IN, DIM_CM, DIM_MM, DIM_PI, DIM_PT, DIM_PX, DIM_PERCENT, DIM_none } UT_Dimension;
 
 /*
@@ -72,6 +76,8 @@ ABI_EXPORT const char * UT_formatDimensionedValue(double value, const char * szU
 ABI_EXPORT bool UT_hasDimensionComponent(const char * sz);
 ABI_EXPORT bool UT_isValidDimensionString(const char * sz, UT_sint32 max_length = 0);
 
+ABI_EXPORT UT_uint32 UT_getDimensionPrecisicion (UT_Dimension dim);
+ABI_EXPORT double UT_getDimensionResolution (UT_Dimension dim);
 
 UT_END_EXTERN_C
 
