@@ -1,5 +1,5 @@
 /* AbiWord
- * Copyright (C) 2002-2003 Hubert Figuiere
+ * Copyright (C) 2002-2005 Hubert Figuiere
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
-/* $Id: xap_CocoaDialog_Utilities.mm,v 1.16.6.1 2005/01/05 22:57:05 fjfranklin Exp $ */
+/* $Id: xap_CocoaDialog_Utilities.mm,v 1.19 2005/03/15 03:25:46 hub Exp $ */
 
 
 
@@ -25,28 +25,26 @@
 
 #include "xap_CocoaDialog_Utilities.h"
 
-/*!
-	Localize a control with a string from a string set
-	
+/** set the label of a NSControl
 	\param control the control AppKit object
-	\param pSS the string set
-	\param stringID the string ID from the string set
+	\param label UT_UTF8String to set. It is a well formed XP label
+	that will be converted.
 	\note if the control object you pass is of an unknown type, the function will
 	NSLog something on the console. Feel free to update the function if you want to
 	handle that kind of object.
- */
-void LocalizeControl (id control, const XAP_StringSet * pSS, XAP_String_Id stringId)
+*/
+void SetNSControlLabel (id control, const UT_UTF8String &label)
 {
-	char buf [1024];
+ 	char buf [1024];
 	NSString*	str;
-	UT_UTF8String label;
-	pSS->getValueUTF8(stringId, label);
+
 	_convertLabelToMac(buf, sizeof (buf), label);
 
 	str = [[NSString alloc] initWithUTF8String:buf];
+
 	if ([control isKindOfClass:[NSButton class]] 
-	     || [control isKindOfClass:[NSBox class]]
-		 || [control isKindOfClass:[NSCell class]]) {
+		|| [control isKindOfClass:[NSBox class]]
+		|| [control isKindOfClass:[NSCell class]]) {
 		[control setTitle:str];
 	}
 	else if ([control isKindOfClass:[NSTabViewItem class]]) {
@@ -64,6 +62,22 @@ void LocalizeControl (id control, const XAP_StringSet * pSS, XAP_String_Id strin
 	[str release];
 }
 
+
+/*!
+	Localize a control with a string from a string set
+	
+	\param control the control AppKit object
+	\param pSS the string set
+	\param stringID the string ID from the string set
+	\seealso SetNSControlLabel
+ */
+void LocalizeControl (id control, const XAP_StringSet * pSS, XAP_String_Id stringId)
+{
+	UT_UTF8String label;
+	pSS->getValueUTF8(stringId, label);
+	SetNSControlLabel(control, label);
+}
+
 /*!
 	Fetch a string from the string set and return a NSString
 	
@@ -73,11 +87,9 @@ void LocalizeControl (id control, const XAP_StringSet * pSS, XAP_String_Id strin
  */
 NSString* LocalizedString (const XAP_StringSet * pSS, XAP_String_Id stringId)
 {
-	char buf [1024];
 	UT_UTF8String label;
 	pSS->getValueUTF8(stringId, label);
-	_convertLabelToMac(buf, sizeof (buf), label);
-	return [NSString stringWithUTF8String:buf];
+	return [NSString stringWithUTF8String:label.utf8_str()];
 }
 
 /*!
