@@ -19,6 +19,12 @@
  * 02111-1307, USA.
  */
 
+/*
+ * Port to Maemo Development Platform 
+ * Author: INdT - Renato Araujo <renato.filho@indt.org.br>
+ */
+
+
 #include <gtk/gtk.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -52,6 +58,11 @@
 #include "gr_UnixImage.h"
 #include "gr_Painter.h"
 
+#ifdef HAVE_HILDON
+#include <hildon-widgets/hildon-file-chooser-dialog.h>
+#endif
+
+
 #include <sys/stat.h>
 
 #include "../../../wp/impexp/xp/ie_types.h"
@@ -61,6 +72,8 @@
 #define PREVIEW_WIDTH  100
 #define PREVIEW_HEIGHT 100
 
+
+
 /*****************************************************************/
 XAP_Dialog * XAP_UnixDialog_FileOpenSaveAs::static_constructor(XAP_DialogFactory * pFactory,
 															 XAP_Dialog_Id id)
@@ -91,7 +104,7 @@ static void s_dialog_response(GtkWidget * /* widget */,
 	{
 		case GTK_RESPONSE_CANCEL:
 		case GTK_RESPONSE_ACCEPT:
-
+		case GTK_RESPONSE_OK:
 			if (answer == GTK_RESPONSE_CANCEL)
 				*ptr = XAP_Dialog_FileOpenSaveAs::a_CANCEL;
 			else
@@ -459,10 +472,7 @@ void XAP_UnixDialog_FileOpenSaveAs::fileTypeChanged(GtkWidget * w)
 	sFileName = sFileName.substr(0,i);
 	sFileName += sSuffix;
 	
-	if (!gtk_file_chooser_select_filename(m_FC,sFileName.c_str()))
-	{
-		gtk_file_chooser_set_current_name(m_FC, UT_basename(sFileName.c_str()));
-	}
+	gtk_file_chooser_set_current_name(m_FC, UT_basename(sFileName.c_str()));
 }
 
 void XAP_UnixDialog_FileOpenSaveAs::onDeleteCancel() 
@@ -562,13 +572,23 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 	XAP_UnixFrameImpl * pUnixFrameImpl = static_cast<XAP_UnixFrameImpl *>(pFrame->getFrameImpl());
 	GtkWidget * parent = pUnixFrameImpl->getTopLevelWindow();
 
+#ifdef HAVE_HILDON
+	GtkWidget * wHildonView = gtk_widget_get_parent(parent);
+
+	m_FC = GTK_FILE_CHOOSER( hildon_file_chooser_dialog_new(GTK_WINDOW(wHildonView),
+							(!m_bSave ? GTK_FILE_CHOOSER_ACTION_OPEN : GTK_FILE_CHOOSER_ACTION_SAVE))
+							);
+	
+#else	
+	
 	m_FC = GTK_FILE_CHOOSER( gtk_file_chooser_dialog_new (szTitle.utf8_str(),
 									GTK_WINDOW(parent),
-									(m_id == XAP_DIALOG_ID_FILE_OPEN || m_id == XAP_DIALOG_ID_INSERT_PICTURE || m_id == XAP_DIALOG_ID_INSERT_FILE ? GTK_FILE_CHOOSER_ACTION_OPEN : GTK_FILE_CHOOSER_ACTION_SAVE),
+									(!m_bSave ? GTK_FILE_CHOOSER_ACTION_OPEN : GTK_FILE_CHOOSER_ACTION_SAVE),
 									GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 									GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
 									NULL)
 							);
+#endif	
 
 	abiSetupModalDialog(GTK_DIALOG(m_FC), pFrame, this, GTK_RESPONSE_ACCEPT);
 	GtkWidget * filetypes_pulldown = NULL;
@@ -764,6 +784,33 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 				g_free (dir);
 				g_free (file);
 			}
+			if(m_id == XAP_DIALOG_ID_FILE_SAVEAS)
+			{
+				const char * szInitialSuffix = UT_pathSuffix(m_szInitialPathname);
+				const char * szSaveTypeSuffix = UT_pathSuffix(m_szSuffixes[m_nDefaultFileType]);
+				if(szInitialSuffix && szSaveTypeSuffix && (UT_strcmp(szInitialSuffix,szSaveTypeSuffix) != 0))
+				{
+					UT_String sFileName = m_szInitialPathname;
+					bool bFoundSuffix = false;
+					UT_sint32 i = 0;
+					for(i= sFileName.length()-1; i> 0; i--)
+					{
+						if(sFileName[i] == '.')
+						{
+							bFoundSuffix = true;
+							break;
+						}
+					}
+					if( bFoundSuffix)
+					{
+						sFileName = sFileName.substr(0,i);
+						UT_String sSuffix = szSaveTypeSuffix;
+						sFileName += sSuffix;
+						FREEP(m_szInitialPathname);
+						UT_cloneString(m_szInitialPathname,sFileName.c_str());
+					}
+				}
+			}
 			gtk_file_chooser_set_filename(m_FC, m_szInitialPathname);
 		}
 		else
