@@ -837,7 +837,7 @@ void PS_Graphics::_emit_DocumentNeededResources(void)
 void PS_Graphics::_emit_IncludeResource(void)
 {
    	UT_Vector vec;
-   	UT_uint32 k,n;
+   	UT_uint32 k;
    	UT_uint32 kLimit = m_vecFontList.getItemCount();
 
 	// fonts are always embedded in the document now, as this was causing way
@@ -989,7 +989,7 @@ void PS_Graphics::_emit_SetColor(void)
   unsigned char newclr;
 
   {
-	  UT_LocaleTransactor(LC_NUMERIC, "C");
+	  UT_LocaleTransactor t(LC_NUMERIC, "C");
 	  switch(m_cs)
 	  {
 	  case GR_Graphics::GR_COLORSPACE_COLOR:
@@ -1104,7 +1104,7 @@ void PS_Graphics::drawRGBImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest)
 			image->width, image->height * -1, image->height);
 	m_ps->writeBytes(buf);
 	
-	int byteWidth = (pPSImage->getHasAlpha() ? 4 : 3);
+	int byteWidth = (pPSImage->hasAlpha() ? 4 : 3);
 
 	g_snprintf(buf, sizeof (buf),"{currentfile rowdata readhexstring pop}\nfalse %d colorimage\n", byteWidth);
 	m_ps->writeBytes(buf);
