@@ -1,5 +1,6 @@
 /* AbiSource Application Framework
  * Copyright (C) 1998 AbiSource, Inc.
+ * BIDI Copyright (c) 2001,2002 Tomas Frydrych
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,11 +23,14 @@
 #define XAP_WIN32APP_H
 
 #include <windows.h>
+#include <tchar.h>
 #include "xap_App.h"
 #include "xap_Win32DialogFactory.h"
 #include "xap_Win32_TB_CFactory.h"
 #include "xap_Strings.h"
 
+#define MAX_CONVBUFFER 256
+
 class XAP_Win32Slurp;
 class XAP_Args;
 class AP_Win32Toolbar_Icons;
@@ -43,7 +47,17 @@ public:
 	XAP_Win32App(HINSTANCE hInstance, XAP_Args * pArgs, const char * szAppName);
 	virtual ~XAP_Win32App(void);
 
-	virtual bool							initialize(void);
+	virtual const char * getDefaultEncoding () const;
+#ifdef UNICODE
+	static const WCHAR * getWideString (const char * p_str);
+	static const char * getUTF8String (const WCHAR * p_str);
+#else
+	static const char * getWideString (const char * p_str);
+	static const char * getUTF8String (const char * p_str);
+#endif
+	
+	
+	virtual bool							initialize(const char * szKeyBindingsKey, const char * szKeyBindingsDefaultValue);
 	virtual XAP_Frame *						newFrame(void) = 0;
 	virtual void							reallyExit(void);
 
@@ -53,7 +67,7 @@ public:
 	virtual XAP_Toolbar_ControlFactory *	getControlFactory(void);
 	virtual const XAP_StringSet *			getStringSet(void) const = 0;
 	virtual const char *					getAbiSuiteAppDir(void) const = 0;
-	virtual void							copyToClipboard(PD_DocumentRange * pDocRange) = 0;
+	virtual void							copyToClipboard(PD_DocumentRange * pDocRange, bool bUseClipboard = true) = 0;
 	virtual void							pasteFromClipboard(PD_DocumentRange * pDocRange, bool, bool) = 0;
 	virtual bool							canPasteFromClipboard(void) = 0;
 	virtual void							cacheCurrentSelection(AV_View *) = 0;
@@ -67,15 +81,35 @@ public:
 	void									enableAllTopLevelWindows(bool);
 	virtual UT_sint32					makeDirectory(const char * szPath, const UT_sint32 mode ) const;
 	virtual UT_sint32 				setupWindowFromPrefs(UT_sint32 iCmdShow, HWND hwndFrame);
+    virtual XAP_App::BidiSupportType        theOSHasBidiSupport() const {return m_eBidiOS;}
+	
+	void									getDefaultGeometry(UT_uint32& width,
+															   UT_uint32& height,
+															   UT_uint32& flags);
+
+	void                                    setHKL(HKL hkl) {m_hkl = hkl;}
+	HKL                                     getHKL()const {return m_hkl;}
+	void                                    setKbdLanguage(HKL hkl);
+
 protected:
 	UT_uint32								_getExeDir(char* pDirBuf, UT_uint32 iBufLen);
 	void									_setAbiSuiteLibDir(void);
+	void									_setBidiOS(void);
+	virtual const char *                    _getKbdLanguage();
 	
 	HINSTANCE								m_hInstance;
 	AP_Win32DialogFactory					m_dialogFactory;
 	AP_Win32Toolbar_ControlFactory			m_controlFactory;
 
 	XAP_Win32Slurp *						m_pSlurp;
+	#ifdef UNICODE
+	static char m_buffer[MAX_CONVBUFFER];
+	static WCHAR m_wbuffer[MAX_CONVBUFFER];
+	#endif
+private:
+	XAP_App::BidiSupportType		        m_eBidiOS;
+	HKL                                     m_hkl; // kbd layout
+												   // handle
 };
 
 #endif /* XAP_WIN32APP_H */
