@@ -39,6 +39,7 @@ enum _XAP_Dialog_Id
 	XAP_DIALOG_ID_FILE_SAVEAS,
 	XAP_DIALOG_ID_FILE_IMPORT,
 	XAP_DIALOG_ID_FILE_EXPORT,
+	XAP_DIALOG_ID_INSERT_FILE,
 	XAP_DIALOG_ID_PRINT,
 	XAP_DIALOG_ID_PRINTPREVIEW,
 	XAP_DIALOG_ID_PRINTTOFILE,			/* the file-save-as spun as pathname-for-print-to-file */
@@ -57,13 +58,30 @@ enum _XAP_Dialog_Id
 	XAP_DIALOG_ID_CLIPART,
 	XAP_DIALOG_ID_ENCODING,
 	XAP_DIALOG_ID_PASSWORD,
+	XAP_DIALOG_ID_IMAGE,
+	XAP_DIALOG_ID_INSERTMATHML,
+	XAP_DIALOG_ID_INSERTOBJECT,
 
+	XAP_DIALOG_ID_HTMLOPTIONS,
 
 #ifdef HAVE_GNOME_DIRECT_PRINT
 	XAP_DIALOG_ID_PRINT_DIRECTLY,	
 #else
-	XAP_DIALOG_ID_PRINT_DIRECTLY = XAP_DIALOG_ID_PRINT, /* just to reduce # of ifdefs */
-#endif	
+#if 0
+	// CANNOT DO THIS !!!
+	// enums are sequential, so this makes everything that comes after
+	// this start again from _ID_PRINT
+	XAP_DIALOG_ID_PRINT_DIRECTLY = XAP_DIALOG_ID_PRINT,
+#else
+#define XAP_DIALOG_ID_PRINT_DIRECTLY XAP_DIALOG_ID_PRINT
+#endif
+#endif
+	XAP_DIALOG_ID_LISTDOCUMENTS,
+	XAP_DIALOG_ID_COMPAREDOCUMENTS,
+	XAP_DIALOG_ID_MERGEDOCUMENTS,
+	XAP_DIALOG_ID_HISTORY,
+	XAP_DIALOG_ID_DOCCOMPARISON,
+	
 	XAP_DIALOG_ID__LAST__				= 1000	/* must be last */
 
 };
