@@ -62,6 +62,8 @@ public:
 	virtual UT_sint32   getY() const;
 	virtual UT_sint32   getWidth() const;
 	virtual UT_sint32   getHeight() const;
+	UT_sint32           getLeftPad(UT_sint32 y, UT_sint32 height);
+	UT_sint32           getRightPad(UT_sint32 y, UT_sint32 height);
 	virtual bool        isVBreakable(void)
 		{ return false;}
 	UT_sint32           getFullX() const;
@@ -91,12 +93,33 @@ public:
 		{m_bIsWrapped = bWrapping;}
 	bool                isWrappingSet(void) const
 		{ return m_bIsWrapped;}
+	void                setTightWrapping( bool bTight)
+	        { m_bIsTightWrapped = bTight;}
+	bool                isTightWrapped(void) const
+	        { return m_bIsTightWrapped;}
+	bool                isTopBot(void) const
+	  { return m_bIsTopBot; }
+	void                setTopBot(bool b)
+	  { m_bIsTopBot = b;}
+	bool                isLeftWrapped(void) const
+	  { return m_bIsLeftWrapped;}
+	void                setLeftWrapped(bool b)
+	  { m_bIsLeftWrapped = b;}
+	bool                isRightWrapped(void) const
+	  { return m_bIsRightWrapped;}
+	void                setRightWrapped(bool b)
+	  { m_bIsRightWrapped = b;}
+
+	bool                overlapsRect(UT_Rect & rec);
+	bool                isAbove(void)
+	        { return m_bIsAbove;}
+	void                setAbove(bool bAbove)
+	        { m_bIsAbove = bAbove;} 
 private:
 	void                   _drawLine (const PP_PropertyMap::Line & style,
 									  UT_sint32 left, UT_sint32 top, 
 									  UT_sint32 right, UT_sint32 bot,
 									  GR_Graphics * pGr);
-	void                   _drawHandleBox(UT_Rect box); 
 	fp_Page * m_pPage;
 	UT_sint32 m_iXpad;
 	UT_sint32 m_iYpad;
@@ -110,8 +133,13 @@ private:
 	PP_PropertyMap::Line   m_lineLeft;
 	PP_PropertyMap::Line   m_lineRight;
 	PP_PropertyMap::Line   m_lineTop;
-    bool                   m_bOverWrote;
-    bool                   m_bIsWrapped;
+	bool                   m_bOverWrote;
+	bool                   m_bIsWrapped;
+	bool                   m_bIsTightWrapped;
+	bool                   m_bIsAbove;
+	bool                   m_bIsTopBot;
+	bool                   m_bIsLeftWrapped;
+	bool                   m_bIsRightWrapped;
 };
 
 
