@@ -33,6 +33,8 @@
 #include "fp_Page.h"
 #include "fp_Column.h"
 
+#include "pd_Iterator.h"
+
 /*!
   Dump fp_Column information
   \param fp File where the dump should be written to
@@ -40,7 +42,7 @@
 void fp_Column::__dump(FILE * fp) const
 {
 	fprintf(fp, "Col: %p X=%d Y=%d W=%d H=%d E=%s\n",
-			(void*) this, getX(), getY(), getWidth(), getHeight(),
+			static_cast<const void*>(this), getX(), getY(), getWidth(), getHeight(),
 			(isEmpty() ? "Yes" : "No"));
 }
 
@@ -54,7 +56,7 @@ void fp_Page::__dump(FILE * fp) const
 {
 	int i, count;
 
-	fprintf(fp,"  Page: %p\n",(void*)this);
+	fprintf(fp,"  Page: %p\n",static_cast<const void*>(this));
 	
 	for (count = countColumnLeaders(), i = 0; i < count; i++)
 	{
@@ -91,7 +93,7 @@ void fp_Run::__dump(FILE * fp) const
 									  "LineBreak", "ColBreak", "PageBreak",
 									  "Field", "FmtMark", "FieldStart",
 									  "FieldEnd", "EOP", "Bookmark",
-									  "Hyperlink" };
+									  "Hyperlink", "DirectionMarker" };
 	UT_ASSERT(NrElements(s_names)==(FPRUN__LAST__-FPRUN__FIRST__));
 	const char * szName = 
 		(((m_iType >= FPRUN__FIRST__) && (m_iType <= FPRUN__LAST__)) 
@@ -103,7 +105,7 @@ void fp_Run::__dump(FILE * fp) const
 		fprintf(fp,"    ");
 
 	fprintf(fp,"Run: %p T=%s Off=%d Len=%d D=%c [x %d y %d w %d h %d]\n",
-			(void*)this, szName, m_iOffsetFirst, m_iLen, 
+			static_cast<const void*>(this), szName, m_iOffsetFirst, m_iLen, 
 			((m_bDirty) ? 'y' : 'n'),
 			m_iX, m_iY, m_iWidth, m_iHeight);
 
@@ -120,26 +122,19 @@ void fp_TextRun::__dump(FILE * fp) const
 	fp_Run::__dump(fp);
 
 	fprintf(fp,"         [");
-	if (m_iLen != 0)
+	if (getLength() != 0)
 	{
-		const UT_UCSChar* pSpan = NULL;
-		UT_uint32 lenSpan = 0;
+		UT_uint32 koff=getBlockOffset();
+		UT_uint32 klen=getLength();
 
-		UT_uint32 koff=m_iOffsetFirst;
-		UT_uint32 klen=m_iLen;
-		
-		while (m_pBL->getSpanPtr(koff, &pSpan, &lenSpan) && (klen > 0))
+		PD_StruxIterator text(getBlock()->getStruxDocHandle(),
+							  koff + fl_BLOCK_STRUX_OFFSET);
+
+		for(UT_uint32 k = 0; k < klen; k++)
 		{
-			UT_uint32 kdraw = MyMin(klen,lenSpan);
-			for (UT_uint32 k=0; k<kdraw; k++)
-			{
-				// a cheap unicode to ascii hack...
-				unsigned char c = (unsigned char)(pSpan[k] & 0x00ff);
-				fprintf(fp,"%c",c);
-			}
-
-			klen -= kdraw;
-			koff += lenSpan;
+			unsigned char c = static_cast<unsigned char>(text[k+fl_BLOCK_STRUX_OFFSET] & 0x00ff);
+			UT_return_if_fail(text.getStatus() == UTIter_OK);
+			fprintf(fp,"%c",c);
 		}
 	}
 	fprintf(fp,"]\n");
@@ -153,7 +148,7 @@ void fp_TextRun::__dump(FILE * fp) const
 void fp_Line::__dump(FILE * fp) const
 {
 	fprintf(fp,"Line: %p Col=%p X=%d Y=%d H=%d W=%d\n",
-			(void*) this, (void*) getContainer(),
+			static_cast<const void*>(this), static_cast<void*>(getContainer()),
 			getX(), getY(), getHeight(), getMaxWidth());
 }
 
