@@ -1,4 +1,4 @@
-/* AbiWord
+ /* AbiWord
  * Copyright (C) 1998,1999 AbiSource, Inc.
  * Copyright (c) 2001,2002 Tomas Frydrych
  *
@@ -62,7 +62,8 @@ fp_TextRun::fp_TextRun(fl_BlockLayout* pBL,
 					   bool bLookupProperties)
 :	fp_Run(pBL,iOffsetFirst, iLen, FPRUN_TEXT),
 	m_fPosition(TEXT_POSITION_NORMAL),
-	m_bSquiggled(false),
+	m_bSpellSquiggled(false),
+	m_bGrammarSquiggled(false),
 	m_pLanguage(NULL),
 	m_bIsOverhanging(false),
 	m_bKeepWidths(false),
@@ -97,6 +98,7 @@ fp_TextRun::fp_TextRun(fl_BlockLayout* pBL,
 
 fp_TextRun::~fp_TextRun()
 {
+        xxx_UT_DEBUGMSG(("!!!!!!!! Text run %x deleted \n",this));
 	DELETEP(m_pRenderInfo);
 	DELETEP(m_pItem);
 }
@@ -246,12 +248,24 @@ void fp_TextRun::_lookupProperties(const PP_AttrProp * pSpanAP,
 	UT_Language lls;
 	const XML_Char * pszLanguage = PP_evalProperty("lang",pSpanAP,pBlockAP,pSectionAP, pDoc, true);
 
+	// NB: m_pLanguage is a pointer into static tables inside UT_Language class and as
+	// such has a guaranteed life-span same as the application; hence no strdup here and
+	// no strcmp later
 	const XML_Char * pszOldLanguage = m_pLanguage;
 	m_pLanguage = lls.getCodeFromCode(pszLanguage);
-	if(pszOldLanguage && m_pLanguage != pszOldLanguage)
+	xxx_UT_DEBUGMSG(("!!!!!!!! Language of run set to %s pointer %x run %x \n",getLanguage(),m_pLanguage,this));
+	if(pszOldLanguage && (m_pLanguage != pszOldLanguage))
 	{
-
-		getBlock()->getDocLayout()->queueBlockForBackgroundCheck(static_cast<UT_uint32>(FL_DocLayout::bgcrSpelling), getBlock());
+	        UT_uint32 reason =  0;
+		if( getBlock()->getDocLayout()->getAutoSpellCheck())
+		{
+		        reason = (UT_uint32) FL_DocLayout::bgcrSpelling;
+		}
+		if( getBlock()->getDocLayout()->getAutoGrammarCheck())
+		{
+		        reason = reason | (UT_uint32) FL_DocLayout::bgcrGrammar;
+		}
+		getBlock()->getDocLayout()->queueBlockForBackgroundCheck(reason, getBlock());
 		bChanged = true;
 	}
 
@@ -346,7 +360,14 @@ bool fp_TextRun::canBreakAfter(void) const
 		PD_StruxIterator text(getBlock()->getStruxDocHandle(),
 							  getBlockOffset() + fl_BLOCK_STRUX_OFFSET);
 		UT_return_val_if_fail(text.getStatus() == UTIter_OK, false);
-		text.setUpperLimit(text.getPosition() + getLength() - 1);
+
+		// in order to allow proper decision on breaking at the end of run, we will try to
+		// set the upper limit one character pass the end of this run
+
+		if(getNextRun())
+			text.setUpperLimit(text.getPosition() + getLength());
+		else
+			text.setUpperLimit(text.getPosition() + getLength() - 1);
 		
 		UT_return_val_if_fail(m_pRenderInfo, false);
 		m_pRenderInfo->m_pText = &text;
@@ -380,7 +401,14 @@ bool fp_TextRun::canBreakBefore(void) const
 							  getBlockOffset() + fl_BLOCK_STRUX_OFFSET );
 
 		UT_return_val_if_fail(text.getStatus() == UTIter_OK, false);
-		text.setUpperLimit(text.getPosition() + getLength() - 1);
+
+		// in order to allow proper decision on breaking at the end of run, we will try to
+		// set the upper limit one character pass the end of this run
+
+		if(getNextRun())
+			text.setUpperLimit(text.getPosition() + getLength());
+		else
+			text.setUpperLimit(text.getPosition() + getLength() - 1);
 		
 		UT_return_val_if_fail(m_pRenderInfo, false);
 		m_pRenderInfo->m_pText = &text;
@@ -523,7 +551,14 @@ bool	fp_TextRun::findMaxLeftFitSplitPoint(UT_sint32 iMaxLeftWidth, fp_RunSplitIn
 						  offset + fl_BLOCK_STRUX_OFFSET);
 
 	m_pRenderInfo->m_pText = &text;
-	text.setUpperLimit(text.getPosition() + getLength() - 1);
+	// in order to allow proper decision on breaking at the end of run, we will try to
+	// set the upper limit one character pass the end of this run
+
+	if(getNextRun())
+		text.setUpperLimit(text.getPosition() + getLength());
+	else
+		text.setUpperLimit(text.getPosition() + getLength() - 1);
+
 	UT_uint32 iPosStart = text.getPosition();
 	
 	//bool bReverse = (getVisDirection() == UT_BIDI_RTL);
@@ -910,7 +945,8 @@ bool fp_TextRun::canMergeWithNext(void)
 		|| (pNext->_getFont() != _getFont())
 		|| (getHeight() != pNext->getHeight())
 		|| (pNext->getField() != getField())
-		|| (pNext->m_pLanguage != m_pLanguage)	//this is not a bug
+		|| (pNext->m_pLanguage != m_pLanguage)	//this is not a bug; see m_pLanguage in
+												//fp_TextRun.h before modifying this line
 		|| (pNext->_getColorFG() != _getColorFG())
 		|| (pNext->_getColorHL() != _getColorHL())
 		|| (pNext->_getColorHL().isTransparent() != _getColorHL().isTransparent())
@@ -975,7 +1011,8 @@ void fp_TextRun::mergeWithNext(void)
 	UT_ASSERT(getDescent() == pNext->getDescent());
 	UT_ASSERT(getHeight() == pNext->getHeight());
 	UT_ASSERT(_getLineWidth() == pNext->_getLineWidth());
-	UT_ASSERT(m_pLanguage == pNext->m_pLanguage); //this is not a bug
+	UT_ASSERT(m_pLanguage == pNext->m_pLanguage); //this is not a bug; see m_pLanguage in
+	                                              //fp_TextRun.h before modifying this line
 	UT_ASSERT(m_fPosition == pNext->m_fPosition);
 	UT_ASSERT(m_iDirOverride == pNext->m_iDirOverride); //#TF
 	//UT_ASSERT(m_iSpaceWidthBeforeJustification == pNext->m_iSpaceWidthBeforeJustification);
@@ -1044,7 +1081,7 @@ void fp_TextRun::mergeWithNext(void)
 	}
 	
 
-	setLength(iMyLen + iNextLen, false);
+	_setLength(iMyLen + iNextLen);
 	_setDirty(isDirty() || pNext->isDirty());
 
 	setNextRun(pNext->getNextRun(), false);
@@ -1109,6 +1146,7 @@ bool fp_TextRun::split(UT_uint32 iSplitOffset)
 	pNew->_setLineWidth(this->_getLineWidth());
 	pNew->_setDirty(true);
 	pNew->m_pLanguage = this->m_pLanguage;
+	xxx_UT_DEBUGMSG(("!!!!--- Run %x gets Language pointer %x \n",pNew,pNew->m_pLanguage));
 	pNew->_setDirection(this->_getDirection()); //#TF
 	pNew->m_iDirOverride = this->m_iDirOverride;
 	// set the visual direction to same as that of the old run
@@ -1257,7 +1295,7 @@ UT_sint32 fp_TextRun::simpleRecalcWidth(UT_sint32 iLength)
     measures widths of individual characters in our draw buffer,
     stores them in the block's width cache and recalculates overall width.
 */
-void fp_TextRun::_measureCharWidths()
+void fp_TextRun::measureCharWidths()
 {
 	_setWidth(0);
 	UT_return_if_fail(m_pRenderInfo);
@@ -1437,6 +1475,11 @@ void fp_TextRun::_clearScreen(bool /* bFullLineHeightRect */)
 					 leftClear,getWidth(),xoff,getLine()->getHeight()));
 
 }
+const XML_Char * fp_TextRun::getLanguage() const
+{ 
+  return m_pLanguage; 
+}
+
 
 void fp_TextRun::_draw(dg_DrawArgs* pDA)
 {
@@ -1831,6 +1874,22 @@ void fp_TextRun::_draw(dg_DrawArgs* pDA)
 		xxx_UT_DEBUGMSG(("_drawText segment %d off %d length %d width %d \n",iSegment,iMyOffset,m_pRenderInfo->m_iLength ,iSegmentWidth[iSegment]));
 		painter.renderChars(*m_pRenderInfo);
 		
+#if 0
+		//DEBUG
+		GR_Font * f = _getFont();
+		UT_uint32 _ascent, _descent, _height;
+		
+		_ascent = pG->getFontAscent(f);
+		_descent = pG->getFontDescent(f);
+		_height = pG->getFontHeight(f);
+		
+		UT_DEBUGMSG(("_drawText font %s ascent = %u height = %u descent = %u\n", f->hashKey().c_str(),
+			_ascent, _height, _descent));
+		painter.drawLine(iX, pDA->yoff - _ascent, iX + iSegmentWidth[iSegment], pDA->yoff - _ascent);
+		painter.drawLine(iX, pDA->yoff, iX + iSegmentWidth[iSegment], pDA->yoff);
+		painter.drawLine(iX, pDA->yoff + _descent, iX + iSegmentWidth[iSegment], pDA->yoff + _descent);
+		//end DEBUG
+#endif		
 		if(iVisDir == UT_BIDI_LTR)
 			iX += iSegmentWidth[iSegment];
 	}
@@ -1846,8 +1905,10 @@ void fp_TextRun::_draw(dg_DrawArgs* pDA)
 	// TODO: draw this underneath (ie, before) the text and decorations
 	if(pG->queryProperties(GR_Graphics::DGP_SCREEN))
 	{
-		m_bSquiggled = false;
-		getBlock()->findSquigglesForRun(this);
+		m_bSpellSquiggled = false;
+		getBlock()->findSpellSquigglesForRun(this);
+		m_bGrammarSquiggled = false;
+		getBlock()->findGrammarSquigglesForRun(this);
 	}
 }
 
@@ -1924,6 +1985,8 @@ void fp_TextRun::_getPartRect(UT_Rect* pRect,
 	m_pRenderInfo->m_iOffset = iStart - getBlockOffset();
 	m_pRenderInfo->m_iLength = iLen;
 	pRect->width = getGraphics()->getTextWidth(*m_pRenderInfo);
+	UT_ASSERT(pRect->left < 10000000);
+	UT_ASSERT(pRect->width >= 0);
 
 	//in case of rtl we are now in the position to calculate the position of the the left corner
 	if(getVisDirection() == UT_BIDI_RTL)
@@ -1944,7 +2007,8 @@ void fp_TextRun::_getPartRect(UT_Rect* pRect,
 		}
 		delete pLRec;
 	}
-	
+	UT_ASSERT(pRect->left < 10000000);
+	UT_ASSERT(pRect->width >= 0);
 }
 
 /*!
@@ -1984,7 +2048,6 @@ bool fp_TextRun::_refreshDrawBuffer()
 		GR_ShapingInfo si(text,iLen, m_pLanguage, iVisDir,
 						  m_pRenderInfo ? m_pRenderInfo->m_eShapingResult : GRSR_Unknown,
 						  _getFont(), m_pItem);
-
 		getGraphics()->shape(si, m_pRenderInfo);
 		
 		UT_ASSERT(m_pRenderInfo && m_pRenderInfo->m_eShapingResult != GRSR_Error );
@@ -2007,7 +2070,7 @@ bool fp_TextRun::_refreshDrawBuffer()
 		_setRefreshDrawBuffer(GRSR_BufferClean);
 
 		// now remeasure our characters
-		_measureCharWidths();
+		measureCharWidths();
 		return true;
 	} //if(m_bRefreshDrawBuffer)
 
@@ -2138,6 +2201,13 @@ void fp_TextRun::_drawFirstChar(bool bSelection)
 	
 	pG->prepareToRenderChars(*m_pRenderInfo);
 	painter.renderChars(*m_pRenderInfo);
+	if(pG->queryProperties(GR_Graphics::DGP_SCREEN))
+	{
+		m_bSpellSquiggled = false;
+		getBlock()->findSpellSquigglesForRun(this);
+		m_bGrammarSquiggled = false;
+		getBlock()->findGrammarSquigglesForRun(this);
+	}
 }
 
 void fp_TextRun::_drawInvisibleSpaces(UT_sint32 xoff, UT_sint32 yoff)
@@ -2194,7 +2264,7 @@ void fp_TextRun::_drawInvisibles(UT_sint32 xoff, UT_sint32 yoff)
 	_drawInvisibleSpaces(xoff,yoff);
 }
 
-void fp_TextRun::_drawSquiggle(UT_sint32 top, UT_sint32 left, UT_sint32 right)
+void fp_TextRun::_drawSquiggle(UT_sint32 top, UT_sint32 left, UT_sint32 right, FL_SQUIGGLE_TYPE iSquiggle)
 {
 	if (!(getGraphics()->queryProperties(GR_Graphics::DGP_SCREEN)))
 	{
@@ -2202,10 +2272,28 @@ void fp_TextRun::_drawSquiggle(UT_sint32 top, UT_sint32 left, UT_sint32 right)
 	}
 
 	GR_Painter painter(getGraphics());
+	if(iSquiggle == FL_SQUIGGLE_SPELL)
+	{
+	  m_bSpellSquiggled = true;
+	}
+	if(iSquiggle == FL_SQUIGGLE_GRAMMAR)
+	{
+	  m_bGrammarSquiggled = true;
+	}
 
-	m_bSquiggled = true;
+	UT_sint32 nPoints = 0;
 
-	UT_sint32 nPoints = getGraphics()->tdu((right - left + getGraphics()->tlu(3))/2);
+	if(iSquiggle == FL_SQUIGGLE_SPELL)
+	{
+	  /* Do /\/\/\/\ */
+
+	  nPoints = getGraphics()->tdu((right - left + getGraphics()->tlu(3))/2);
+	}
+	else
+	{
+	  // Do _|-|_|-|
+	  nPoints = getGraphics()->tdu((right - left + getGraphics()->tlu(3)));
+	}  
 	UT_return_if_fail(nPoints >= 1); //can be 1 for overstriking chars
 
 	/*
@@ -2228,19 +2316,55 @@ void fp_TextRun::_drawSquiggle(UT_sint32 top, UT_sint32 left, UT_sint32 right)
 	points[0].y = top;
 
 	bool bTop = false;
-
-	for (UT_sint32 i = 1; i < nPoints; i++, bTop = !bTop)
+	if(iSquiggle ==  FL_SQUIGGLE_SPELL)
 	{
+	  for (UT_sint32 i = 1; i < nPoints; i++, bTop = !bTop)
+	  {
 		points[i].x = points[i-1].x + getGraphics()->tlu(2);
 		points[i].y = (bTop ? top : top + getGraphics()->tlu(2));
-	}
+	  }
 
-	if (points[nPoints-1].x > right)
+	  if (points[nPoints-1].x > right)
+	  {
+	    points[nPoints-1].x = right;
+	    points[nPoints-1].y = top + getGraphics()->tlu(1);
+	  }
+	}
+	else
 	{
-		points[nPoints-1].x = right;
-		points[nPoints-1].y = top + getGraphics()->tlu(1);
-	}
 
+	  UT_return_if_fail(nPoints >= 2); //can be 1 for overstriking chars
+	  points[0].x = left;
+	  points[0].y = top + getGraphics()->tlu(2);
+	  UT_sint32 i =0;
+	  for (i = 1; i < nPoints-2; i +=2)
+	  {
+		points[i].x = points[i-1].x + getGraphics()->tlu(2);
+		points[i].y = (bTop ? top : top + getGraphics()->tlu(2)) ;
+		points[i+1].x = points[i].x;
+		points[i+1].y = (bTop ? top + getGraphics()->tlu(2) : top);
+		bTop = ! bTop;
+	  }
+	  if(i == (nPoints-2))
+	  {
+		points[i].x = points[i-1].x + getGraphics()->tlu(2);
+		points[i].y = (bTop ? top : top + getGraphics()->tlu(2)) ;
+		points[i+1].x = points[i].x;
+		points[i+1].y = (bTop ? top + getGraphics()->tlu(2) : top);
+		bTop = ! bTop;
+	  }
+	  else if( i == (nPoints-1))
+	  {
+	    points[nPoints-1].x = right;
+	    points[i].y = (bTop ? top : top + getGraphics()->tlu(2)) ;
+	  }
+	  if (points[nPoints-1].x > right)
+	  {
+	    points[nPoints-1].x = right;
+	    points[i].y = (bTop ? top : top + getGraphics()->tlu(2)) ;
+	  }
+
+	}
 	getGraphics()->setLineProperties(getGraphics()->tluD(1.0),
 									 GR_Graphics::JOIN_MITER,
 									 GR_Graphics::CAP_PROJECTING,
@@ -2251,7 +2375,7 @@ void fp_TextRun::_drawSquiggle(UT_sint32 top, UT_sint32 left, UT_sint32 right)
 	if (points != scratchpoints) delete[] points;
 }
 
-void fp_TextRun::drawSquiggle(UT_uint32 iOffset, UT_uint32 iLen)
+void fp_TextRun::drawSquiggle(UT_uint32 iOffset, UT_uint32 iLen,FL_SQUIGGLE_TYPE iSquiggle )
 {
 //	UT_ASSERT(iLen > 0);
 	if (iLen == 0)
@@ -2266,20 +2390,29 @@ void fp_TextRun::drawSquiggle(UT_uint32 iOffset, UT_uint32 iLen)
 	UT_sint32 xoff = 0, yoff = 0;
 	UT_sint32 iAscent = getLine()->getAscent();
 	UT_sint32 iDescent = getLine()->getDescent();
-
+	if(iOffset < getBlockOffset())
+	{
+	  iOffset = getBlockOffset();
+	}
 	// we'd prefer squiggle to leave one pixel below the baseline,
 	// but we need to force all three pixels inside the descent
 	// we cannot afford the 1pixel gap, it leave dirt on screen -- Tomas
 	UT_sint32 iGap = (iDescent > 3) ?/*1*/0 : (iDescent - 3);
-
-	getGraphics()->setColor(_getView()->getColorSquiggle());
+	if(iSquiggle == FL_SQUIGGLE_GRAMMAR)
+	{
+	  //	  iGap += getGraphics()->tlu(2);
+	}
+	getGraphics()->setColor(_getView()->getColorSquiggle(iSquiggle));
 
 	getLine()->getScreenOffsets(this, xoff, yoff);
 
 	UT_Rect r;
 	_getPartRect( &r, xoff, yoff, iOffset, iLen);
-
-	_drawSquiggle(r.top + iAscent + iGap + getGraphics()->tlu(1), r.left, r.left + r.width);
+	if(r.width > getWidth())
+	{
+	  r.width = getWidth();
+	}
+	_drawSquiggle(r.top + iAscent + iGap + getGraphics()->tlu(1), r.left, r.left + r.width,iSquiggle);
 	xxx_UT_DEBUGMSG(("Done draw sqiggle for run in block %x \n",getBlock()));
 }
 
@@ -2467,6 +2600,21 @@ void fp_TextRun::justify(UT_sint32 iAmount, UT_uint32 iSpacesInRun)
 		m_pRenderInfo->m_iLength = getLength();
 	
 		_setWidth(getWidth() + iAmount);
+
+#ifdef XP_UNIX_TARGET_GTK
+		// Because Pango does not yet support justification, we need to iterate over the raw
+		// text counting spaces; this is not required by the default graphics class, nor
+		// Uniscribe and for performance reasons it is therefore only conditionally compiled
+		// in; once Pango supports justification, this will not be needed, so this whole lot
+		// will be again removed
+		UT_uint32 iPosStart = getBlockOffset() + fl_BLOCK_STRUX_OFFSET;
+		PD_StruxIterator text(getBlock()->getStruxDocHandle(),iPosStart);
+		text.setUpperLimit(iPosStart + getLength() - 1);
+		m_pRenderInfo->m_pText = & text;
+#else
+		m_pRenderInfo->m_pText = NULL;
+#endif
+
 		m_pRenderInfo->m_iJustificationPoints = iSpacesInRun;
 		m_pRenderInfo->m_iJustificationAmount = iAmount;
 		getGraphics()->justify(*m_pRenderInfo);
@@ -2485,6 +2633,22 @@ UT_sint32 fp_TextRun::countJustificationPoints(bool bLast) const
 	UT_return_val_if_fail(m_pRenderInfo, 0);
 	m_pRenderInfo->m_iLength = getLength();
 
+	UT_return_val_if_fail(m_pRenderInfo->m_iLength > 0, 0);
+
+#ifdef XP_UNIX_TARGET_GTK
+	// Because Pango does not yet support justification, we need to iterate over the raw
+	// text counting spaces; this is not required by the default graphics class, nor
+	// Uniscribe and for performance reasons it is therefore only conditionally compiled
+	// in; once Pango supports justification, this will not be needed, so this whole lot
+	// will be again removed
+	UT_uint32 iPosStart = getBlockOffset() + fl_BLOCK_STRUX_OFFSET;
+	PD_StruxIterator text(getBlock()->getStruxDocHandle(),iPosStart);
+	text.setUpperLimit(iPosStart + getLength() - 1);
+	m_pRenderInfo->m_pText = & text;
+#else
+	m_pRenderInfo->m_pText = NULL;
+#endif
+	
 	m_pRenderInfo->m_bLastOnLine = bLast;
 	return getGraphics()->countJustificationPoints(*m_pRenderInfo);
 }
