@@ -236,6 +236,7 @@ public:
 	virtual void        updateLayout(void);
 	virtual void        rebuildLayout(void);
 	virtual void        remeasureCharsWithoutRebuild();
+	virtual void        fontMetricsChange();
 	virtual bool		isSelectionEmpty(void) const;
 	bool                isSelectAll(void)
 	{ return m_Selection.isSelectAll();}
@@ -717,6 +718,7 @@ public:
 	FV_BIDI_Order	    getBidiOrder()const {return m_eBidiOrder;}
 	void                setBidiOrder(FV_BIDI_Order o) {m_eBidiOrder = o;}
 
+	bool                isMathSelected(UT_sint32 x, UT_sint32 y, PT_DocPosition & pos);
 	// -- plugins
         bool                isMathLoaded(void);
 	bool                isGrammarLoaded(void);
