@@ -338,6 +338,21 @@ void FV_VisualDragText::mouseDrag(UT_sint32 x, UT_sint32 y)
 		      m_pView->setSelectionMode(FV_SelectionMode_TableRow);
 		}
 
+	}
+	if(!m_bNotDraggingImage && (expX.height > 0))
+	{
+	  xxx_UT_DEBUGMSG(("expY left %d top %d width %d height %d \n",expX.left,expX.top,expX.width,expX.height));
+		getGraphics()->setClipRect(&expX);
+		if(m_bSelectedRow);
+		{
+		      m_pView->setSelectionMode(FV_SelectionMode_NONE);
+		}
+		m_pView->updateScreen(false);
+		if(m_bSelectedRow);
+		{
+		      m_pView->setSelectionMode(FV_SelectionMode_TableRow);
+		}
+
 	}
 	if(!m_bNotDraggingImage)
 	{
