@@ -54,12 +54,17 @@ public:
 	void                  mouseCopy(UT_sint32 x, UT_sint32 y);
 	void                  mouseRelease(UT_sint32 x, UT_sint32 y);
 	void                  drawImage(void);
-	void                  getImageFromSelection(UT_sint32 x, UT_sint32 y); 	
+	void                  getImageFromSelection(UT_sint32 x, UT_sint32 y);
+	bool                  reposOffsets(UT_sint32 x, UT_sint32 y);
 	PT_DocPosition        getPosFromXY(UT_sint32 x, UT_sint32 y);
 	void                  drawCursor(PT_DocPosition newPos);
 	static void 		  _autoScroll(UT_Worker * pTimer);
 	void                  clearCursor(void);
-
+	void                  abortDrag(void);
+	bool                  isNotdraggingImage(void)
+	  { return m_bNotDraggingImage;}
+	bool                  isDoingCopy(void)
+	  { return m_bDoingCopy;}
 private:
 	FV_View *             m_pView;
 	FV_VisualDragMode     m_iVisualDragMode;
@@ -82,6 +87,8 @@ private:
 	UT_sint32			  m_yLastMouse;
 
 	bool                  m_bDoingCopy;
+	bool                  m_bNotDraggingImage;
+	bool                  m_bSelectedRow;
 };
 
 #endif /* FV_VISUALDRAGTEXT_H */
