@@ -39,8 +39,9 @@
 #include "pd_Document.h"
 
 
-
 /****************************************************************/
+
+/// This is the sole explicit constructor of class PP_AttrProp.
 PP_AttrProp::PP_AttrProp()
 {
 	m_pAttributes = NULL;
@@ -54,6 +55,7 @@ PP_AttrProp::PP_AttrProp()
 	m_bRevisionHidden = false;
 }
 
+/// This is the sole explicit destructor of class PP_AttrProp.
 PP_AttrProp::~PP_AttrProp()
 {
 	xxx_UT_DEBUGMSG(("deleting pp_AttrProp %x \n",this));
@@ -258,6 +260,7 @@ bool	PP_AttrProp::setAttribute(const XML_Char * szName, const XML_Char * szValue
 			if (!*q)
 			{
 				free(pOrig);
+				UT_DEBUGMSG(("props: %s\n", szValue));
 				return false;
 			}
 
@@ -290,8 +293,22 @@ bool	PP_AttrProp::setAttribute(const XML_Char * szName, const XML_Char * szValue
 		free(pOrig);
 		return true;
 	}
-	else								// not "PROPS" -- add to attribute list
+	else if (0 == strcmp(szName, PT_XID_ATTRIBUTE_NAME) && *szValue)
 	{
+		// XID is a unique id for the xml element / PT frag. Its function is to facilitate
+		// comparing/merging documents and we do not want it in the AP
+		return true;
+	}
+	else // not "PROPS" -- add to attribute list
+	{
+		UT_UTF8String url;
+		if (szValue && *szValue && (0 == strcmp(szName, "xlink:href") || 0 == strcmp(szName, "href")))
+		{
+			url = szValue;
+			url.decodeURL();
+			szValue = url.utf8_str();
+		}
+		
 		if (!m_pAttributes)
 		{
 			m_pAttributes = new UT_GenericStringMap<XML_Char*>(5);
@@ -315,6 +332,10 @@ bool	PP_AttrProp::setAttribute(const XML_Char * szName, const XML_Char * szValue
 		UT_lowerString(copy);
 		char * szDupValue = szValue ? UT_strdup(szValue) : NULL;
 
+		// get rid of any illegal chars we might have been given
+		UT_validXML(copy);
+		UT_validXML(szDupValue);
+
 		const char * pEntry = m_pAttributes->pick(copy);
 
 		if(pEntry)
@@ -339,8 +360,17 @@ bool	PP_AttrProp::setAttribute(const XML_Char * szName, const XML_Char * szValue
 	}
 }
 
+/*! This method inserts a new pair of property name and value into [this] APs set of
+	 properties, creating "props" if it does not already exist and overwriting the value
+	 of any property of the same name with the newly passed value.
+	 (?)It appears as though we replace the entire pair, rather than only the value.
+		 (?)Is there a reason for this?
+	 \return Whether or not the operation succeeded.
+*/
 bool	PP_AttrProp::setProperty(const XML_Char * szName, const XML_Char * szValue)
 {
+	UT_return_val_if_fail( szName, false );
+	
 	if (!m_pProperties)
 	{
 		m_pProperties = new UT_GenericStringMap<PropertyPair*>(5);
@@ -357,6 +387,24 @@ bool	PP_AttrProp::setProperty(const XML_Char * szName, const XML_Char * szValue)
 	// be present
 	// 
 	//bool bRemove = (!szValue || !*szValue);
+
+	// get rid of any chars invalid in xml
+	char * szName2 = NULL;
+	if(!UT_isValidXML(szName))
+	{
+		szName2 = UT_strdup(szName);
+
+		// get rid of any illegal chars we were passed
+		UT_validXML(szName2);
+
+		szName = szName2;
+	}
+	
+	char * szValue2 = szValue ? UT_strdup(szValue) : NULL;
+	UT_return_val_if_fail( szName && (szValue2 || !szValue), false);
+
+	// get rid of any illegal chars we might have been given in the value
+	UT_validXML(szValue2);
 	
 	const PropertyPair * pEntry = m_pProperties->pick(szName);
 	if (pEntry)
@@ -375,17 +423,28 @@ bool	PP_AttrProp::setProperty(const XML_Char * szName, const XML_Char * szValue)
 		if (p->second())
 			delete p->second();
 		delete p;
-		m_pProperties->set(szName, new PropertyPair(szValue ? UT_strdup(szValue) : NULL, NULL));
+		m_pProperties->set(szName, new PropertyPair(szValue2, NULL));
 	}
 	else
 	{
-		m_pProperties->insert(szName, new PropertyPair(szValue ? UT_strdup(szValue) : NULL, NULL));
+		m_pProperties->insert(szName, new PropertyPair(szValue2, NULL));
 	}
+
+	// free the name duplicate if necessary
+	FREEP(szName2);
+	
 	return true;
 }
 
-/*!
-    Always check the return value before trying to work with szValue !!!
+/*! This method finds the Nth attribute where N is
+	 \param ndx The number in order of the attribute to be found
+	 and assigns that attribute's name and value to
+	 \param szName The name of the attribute found
+	 \param szValue The value of the attribute found
+	 respectively.  The method returns
+	 \return whether or not the operation succeeded.
+
+    WARNING: Always check the return value before trying to work with szValue!
 */
 bool	PP_AttrProp::getNthAttribute(int ndx, const XML_Char *& szName, const XML_Char *& szValue) const
 {
@@ -416,8 +475,15 @@ bool	PP_AttrProp::getNthAttribute(int ndx, const XML_Char *& szName, const XML_C
 	return false;
 }
 
-/*!
-    Always check the return value before trying to work with szValue !!!
+/*! This method finds the Nth property where N is
+	 \param ndx The number in order of the property to be found
+	 and assigns that property's name and value to
+	 \param szName The name of the property found
+	 \param szValue The value of the property found
+	 respectively.  The method returns
+	 \return whether or not the operation succeeded.
+
+    WARNING: Always check the return value before trying to work with szValue!
 */
 bool	PP_AttrProp::getNthProperty(int ndx, const XML_Char *& szName, const XML_Char *& szValue) const
 {
@@ -449,6 +515,16 @@ bool	PP_AttrProp::getNthProperty(int ndx, const XML_Char *& szName, const XML_Ch
 	return false;
 }
 
+/*! This method finds the property indicated by name
+	 \param szName (the name of the property the value of which to find)
+	 and assigns its value to
+	 \param szValue (the value found of the property requested)
+	 or returns false if the properties as a whole or the property requested are not found.
+	 It returns
+	 \return whether or not the operation succeeded.
+
+	 WARNING: Be sure to check the return value before trying to work with szValue.
+*/
 bool PP_AttrProp::getProperty(const XML_Char * szName, const XML_Char *& szValue) const
 {
 	if (!m_pProperties)
@@ -461,9 +537,11 @@ bool PP_AttrProp::getProperty(const XML_Char * szName, const XML_Char *& szValue
 	szValue = pEntry->first();
 	return true;
 }
-/*!
- * Do not free this memory. It's cached here.
- */
+/*! This method retrieves the entirety of [this] AP's "props", and returns it as an array of
+	 XML_Char * pairs (name and value).
+
+    WARNING: Do not free this memory. It's cached here.
+*/
 const XML_Char ** PP_AttrProp::getProperties () const
 {
 	if(!m_pProperties)
@@ -491,7 +569,8 @@ const XML_Char ** PP_AttrProp::getProperties () const
 	return m_szProperties;
 }
 
-
+/*! (?)TODO: PLEASE DOCUMENT ME!
+*/
 const PP_PropertyType *PP_AttrProp::getPropertyType(const XML_Char * szName, tProperty_type Type) const
 {
 	if (!m_pProperties)
@@ -512,6 +591,16 @@ const PP_PropertyType *PP_AttrProp::getPropertyType(const XML_Char * szName, tPr
 	return pEntry->second();
 }
 
+/*! This method finds the attribute indicated by name
+	 \param szName (the name of the attribute the value of which to find)
+	 and assigns its value to
+	 \param szValue (the value found of the attribute requested)
+	 or returns false if the attributes as a whole or the attribute requested are not found.
+	 It returns
+	 \return whether or not the operation succeeded.
+
+	 WARNING: Be sure to check the return value before trying to work with szValue.
+*/
 bool PP_AttrProp::getAttribute(const XML_Char * szName, const XML_Char *& szValue) const
 {
 	if (!m_pAttributes)
@@ -529,6 +618,17 @@ bool PP_AttrProp::getAttribute(const XML_Char * szName, const XML_Char *& szValu
 	return true;
 }
 
+/// Returns whether or not the AP has any properties.
+/*! This method checks [this] AP for the "props" attribute.
+	 The "props" attribute is a special attribute that contains
+	 properties.  If the "props" attribute is absent, the
+	 method returns false.  If the "props" attribute is present
+	 but empty (m_pProperties->size() == 0), it returns false.
+	 If the "props" attribute is present and has something in it,
+	 (m_pProperties->size() > 0), it returns true, but beware that
+	 no sanity checking is done to make sure that whatever is in
+	 there is anything more legible than uninitialized memory.
+*/
 bool PP_AttrProp::hasProperties(void) const
 {
 	if (!m_pProperties)
@@ -537,6 +637,15 @@ bool PP_AttrProp::hasProperties(void) const
 	return (m_pProperties->size() > 0);
 }
 
+/// Returns whether or not the AP has any attributes.
+/*! This method checks [this] AP for the presence of any attribute.
+	 If there is any attribute at all in the AP, m_pAttributes->size()
+	 returns positive and so does this method.  Otherwise, this method
+	 returns false.
+	 Beware that no sanity checking is done to make sure that whatever
+	 is in there (being counted by size()) is anything more legible
+	 than uninitialized memory.
+*/
 bool PP_AttrProp::hasAttributes(void) const
 {
 	if (!m_pAttributes)
@@ -545,11 +654,15 @@ bool PP_AttrProp::hasAttributes(void) const
 	return (m_pAttributes->size() > 0);
 }
 
+/// Returns whether or not the given attributes and properties are identically present in [this] AP.
+/*! This method compares the given attributes and properties with those already present.
+	 It compares the given items as inseparable pairs - if the attribute or property is
+	 present in name but contains a different value, that does not count and false is
+	 returned.
+	 \return A bool indicating (directly) both presence and equality.
+*/
 bool PP_AttrProp::areAlreadyPresent(const XML_Char ** attributes, const XML_Char ** properties) const
 {
-	// return TRUE if each attribute and property is already present
-	// and has the same value as what we have.
-
 	if (attributes && *attributes)
 	{
 		const XML_Char ** p = attributes;
@@ -559,13 +672,32 @@ bool PP_AttrProp::areAlreadyPresent(const XML_Char ** attributes, const XML_Char
 			    It seems that we also want empty strings for attributes,
 				at least for the 'param' attribute which goes with fields.
 				Is this bogus too? -PL
+
+				Yes. 
+				We use empty strings and NULL in values to indicate that the
+				attribute/property should be absent. Sometimes these values filter down
+				here, so we need to handle this. TF
 			*/
-			UT_return_val_if_fail (p[1] /* && *p[1]*/, false);	// require value for each name
+			// UT_return_val_if_fail (p[1] /* && *p[1]*/, false);	// require value for each name
+
+			// first deal with the case where the value is set to NULL or "" -- we want
+			// that attribute to be absent, not present
 			const XML_Char * szValue = NULL;
-			if (!getAttribute(p[0],szValue))
-				return false;		// item not present
-			if (strcmp(p[1],szValue)!=0)
-				return false;		// item has different value
+
+			if((!p[1] || !*p[1]) && getAttribute(p[0],szValue) && szValue && *szValue)
+				return false;
+			// the 'props' attribute has to be handled separatedly, since it is not
+			// returned using getAttribute() (it is not stored as attribute)
+			else if((!p[1] || !*p[1]) && !strcmp(p[0],"props") && hasProperties())
+				return false;
+			else if(p[1] && *p[1])
+			{
+				if (!getAttribute(p[0],szValue))
+					return false;		// item not present
+				if (strcmp(p[1],szValue)!=0)
+					return false;		// item has different value
+			}
+					
 			p += 2;
 		}
 	}
@@ -579,13 +711,27 @@ bool PP_AttrProp::areAlreadyPresent(const XML_Char ** attributes, const XML_Char
 				Jeff, I weakened the following assert because we
 				*want* to represent no tabstops as an empty string.
 				If this isn't safe, let me know.   -- PCR
+
+				We use empty strings and NULL in values to indicate that the
+				attribute/property should be absent. Sometimes these values filter down
+				here, so we need to handle this. TF
 			*/
-			UT_return_val_if_fail (p[1] /* && *p[1] */, false);	// require value for each name
+			// UT_return_val_if_fail (p[1] /* && *p[1]*/, false);	// require value for each name
+
+			// first deal with the case where the value is set to NULL or "" -- we want
+			// that attribute to be absent, not present
 			const XML_Char * szValue = NULL;
-			if (!getProperty(p[0],szValue))
-				return false;		// item not present
-			if (strcmp(p[1],szValue)!=0)
-				return false;		// item has different value
+
+			if((!p[1] || !*p[1]) && getProperty(p[0],szValue) && szValue && *szValue)
+				return false;
+			else if(p[1] && p[1])
+			{
+				if (!getProperty(p[0],szValue))
+					return false;		// item not present
+				if (strcmp(p[1],szValue)!=0)
+					return false;		// item has different value
+			}
+			
 			p += 2;
 		}
 	}
@@ -593,12 +739,14 @@ bool PP_AttrProp::areAlreadyPresent(const XML_Char ** attributes, const XML_Char
 	return true;						// everything matched
 }
 
+/*! Find out if any attribute- or property-name is present.
+	 This is like areAlreadyPresent(), but we don't care about
+	 the values, and it returns true after the first discovery
+	 regardless of whether or not any other of the given names are present.
+	 \return A bool that's TRUE if any of the given attr. or prop. names is present, false otherwise.
+*/
 bool PP_AttrProp::areAnyOfTheseNamesPresent(const XML_Char ** attributes, const XML_Char ** properties) const
 {
-	// return TRUE if any attribute- or property-name is present.
-	// this is like areAlreadyPresent() but we don't care about
-	// the values.
-
 	// TODO consider using the fact that we are now (Dec 12 1998) using
 	// TODO alpha-hash-table rather than just a hash-table to optimize
 	// TODO these loops somewhat.
@@ -630,16 +778,20 @@ bool PP_AttrProp::areAnyOfTheseNamesPresent(const XML_Char ** attributes, const
 	return false;					// didn't find any
 }
 
+/*! Checks to see if the given AP is identical to itself ([this]).  It also contains
+	 some useful points of instrumentation for benchmarking table and usage characteristics.
+	 \return TRUE, if and only if we match the AP given, false otherwise.
+*/
 bool PP_AttrProp::isExactMatch(const PP_AttrProp * pMatch) const
 {
+	// The counters below are used in testing to profile call and chksum characteristics,
+	// including collision rates.
+	// NB: I'm not sure this initialization block is in the correct place.
 #ifdef PT_TEST
 	static UT_uint32 s_Calls = 0;
 	static UT_uint32 s_PassedCheckSum = 0;
 	static UT_uint32 s_Matches = 0;
 #endif
-
-	// return TRUE iff we exactly match the AP given.
-
 #ifdef PT_TEST
 	s_Calls++;
 #endif
@@ -687,7 +839,7 @@ bool PP_AttrProp::isExactMatch(const PP_AttrProp * pMatch) const
 
 			v1 = ca1.next();
 			v2 = ca2.next();
-		} while (ca1.is_valid());
+		} while (ca1.is_valid() && ca2.is_valid());
 	}
 
 	if (countMyProps > 0)
@@ -714,7 +866,7 @@ bool PP_AttrProp::isExactMatch(const PP_AttrProp * pMatch) const
 
 			v1 = cp1.next();
 			v2 = cp2.next();
-		} while (cp1.is_valid());
+		} while (cp1.is_valid() && cp2.is_valid());
 
 	#ifdef PT_TEST
 		s_Matches++;
@@ -724,15 +876,14 @@ bool PP_AttrProp::isExactMatch(const PP_AttrProp * pMatch) const
 	return true;
 }
 
+/*! Create a new AttrProp based upon the given one, adding or replacing the items given.
+	 \return NULL on failure, the newly-created PP_AttrProp clone otherwise.
+*/
 PP_AttrProp * PP_AttrProp::cloneWithReplacements(const XML_Char ** attributes,
 												 const XML_Char ** properties,
 												 bool bClearProps) const
 {
 	bool bIgnoreProps = false; // see below
-	
-	// create a new AttrProp based upon the given one
-	// and adding or replacing the items given.
-	// return NULL on failure.
 
 	// first, create a new AttrProp using just the values given.
 
@@ -798,9 +949,7 @@ Failed:
 	return NULL;
 }
 
-/*
-  This function will remove all properties that are set to ""
-*/
+/// This function will remove all properties that are set to ""
 void PP_AttrProp::_clearEmptyProperties()
 {
 	if(!m_pProperties)
@@ -832,13 +981,14 @@ void PP_AttrProp::_clearEmptyProperties()
 	}
 }
 
+/// This method clears both empty attributes and empty properties from [this] AP.
 void PP_AttrProp::prune()
 {
 	_clearEmptyAttributes();
 	_clearEmptyProperties();
 }
 
-
+/// This function will remove all attributes that are equal to "" (*<XML_Char *> == NULL)
 void PP_AttrProp::_clearEmptyAttributes()
 {
 	if(!m_pAttributes)
@@ -858,15 +1008,17 @@ void PP_AttrProp::_clearEmptyAttributes()
 	}
 }
 
+/*! Create a new AttrProp based upon the given one, removing the items given
+	 regardless of their value.  See also PP_AttrProp::cloneWithEliminationIfEqual,
+	 which does similarly but only removes the items given if their value is equal
+	 to the value given.
+	 \return NULL on failure, the newly-created PP_AttrProp clone otherwise.
+*/
 PP_AttrProp * PP_AttrProp::cloneWithElimination(const XML_Char ** attributes,
 												const XML_Char ** properties) const
 {
-	// create a new AttrProp based upon the given one
-	// and removing the items given.
-	// return FALSE on failure.
 
 	// first, create an empty AttrProp.
-
 	PP_AttrProp * papNew = new PP_AttrProp();
 	if (!papNew)
 		goto Failed;
@@ -935,6 +1087,11 @@ Failed:
 	return NULL;
 }
 
+/*! After the construction of a new AP, use this method to mark it read-only.
+	 It returns if the AP is already read-only; otherwise it marks [this] AP
+	 as read-only and then computes its checksum in order to speed subsequent
+	 equivalence testing.  There is no return value.
+*/
 void PP_AttrProp::markReadOnly(void)
 {
 	UT_return_if_fail (!m_bIsReadOnly);
@@ -942,15 +1099,16 @@ void PP_AttrProp::markReadOnly(void)
 	_computeCheckSum();
 }
 
+/*! Create a new AttrProp based upon the given one, removing the items given
+	 if their value is equal to that given.  See also PP_AttrProp::cloneWithElimination,
+	 which does similarly but removes the items given regardless of whether or not
+	 their value is equal to the value given.
+	 \return NULL on failure, the newly-created PP_AttrProp clone otherwise.
+*/
 PP_AttrProp * PP_AttrProp::cloneWithEliminationIfEqual(const XML_Char ** attributes,
 												const XML_Char ** properties) const
 {
-	// create a new AttrProp based upon the given one
-	// and removing the items given.
-	// return FALSE on failure.
-
 	// first, create an empty AttrProp.
-
 	PP_AttrProp * papNew = new PP_AttrProp();
 	if (!papNew)
 		goto Failed;
@@ -1020,7 +1178,8 @@ Failed:
 	return NULL;
 }
 
-  
+/*! (?)TODO: PLEASE DOCUMENT ME!
+*/
 static UT_uint32 hashcodeBytesAP(UT_uint32 init, const void * pv, UT_uint32 cb)
 {
  	// modified from ut_string_class.cpp's hashcode() which got it from glib
@@ -1041,7 +1200,10 @@ static UT_uint32 hashcodeBytesAP(UT_uint32 init, const void * pv, UT_uint32 cb)
  	return h;
 }
 
-
+/*! Compute the checksum by which we speed AP equivalence testing.  (?)To the best of my knowledge,
+	 collision is still possible.  (?)TODO: Document the algorithm/process here, and
+	 answer remaining questions about this chunk of code.
+*/
 void PP_AttrProp::_computeCheckSum(void)
 {
 	m_checkSum = 0;
@@ -1115,6 +1277,13 @@ void PP_AttrProp::_computeCheckSum(void)
 	return;
 }
 
+/*! This is an accessor method that gets the checksum of [this] AP, by which we speed
+	 equivalence testing.  The speedup occurs when we bypass full-testing
+	 in the case of checksum mismatch when we know for sure that the APs are not
+	 equivalent.  Because collisions still occur, we do the full-testing if the
+	 checksums do match.
+	 \retval m_checkSum The unsigned 32-bit integer which serves as the AP's checksum.
+*/
 UT_uint32 PP_AttrProp::getCheckSum(void) const
 {
 	UT_ASSERT_HARMLESS(m_bIsReadOnly);
@@ -1165,10 +1334,15 @@ void PP_AttrProp::setIndex(UT_uint32 i)
 	m_szProperties = NULL;
 }
 
-/*!
-    in contrast to is isExactMatch this function will return true if
+/*! This method checks if [this] AP and the given AP are functionally equivalent.
+    In contrast to is isExactMatch this function will return true if
     the attrs and props are same even if they are in different order;
-    it is computationally much more involved than isExactMatch()
+    it is computationally much more involved than isExactMatch().
+	 On that note, it may be worth looking into putting some more explicit collision
+	 guarantees into the checksum computation algorithm, such to take advantage of
+	 those checksums here.  IOW, make it such to be trusted that if checksums match,
+	 APs are equivalent (but not necessarily exact matches).
+	 \retval TRUE if equivalent to given AP (regardless of order), FALSE otherwise.
 */
 bool PP_AttrProp::isEquivalent(const PP_AttrProp * pAP2) const
 {
@@ -1190,11 +1364,11 @@ bool PP_AttrProp::isEquivalent(const PP_AttrProp * pAP2) const
 			return false;
 
 		// ignore property attribute
-		if(0 != UT_strcmp(pValue, PT_PROPS_ATTRIBUTE_NAME))
+		if(0 == UT_strcmp(pValue, PT_PROPS_ATTRIBUTE_NAME))
 			continue;
 
 		// handle revision attribute correctly
-		if(0 != UT_strcmp(pValue, PT_REVISION_ATTRIBUTE_NAME))
+		if(0 == UT_strcmp(pValue, PT_REVISION_ATTRIBUTE_NAME))
 		{
 			// requires special treatment
 			PP_RevisionAttr r1(pValue);
@@ -1223,6 +1397,11 @@ bool PP_AttrProp::isEquivalent(const PP_AttrProp * pAP2) const
 	return true;
 }
 
+/*! This method does the same as PP_AttrProp::isEquivalent(const PP_AttrProp *) except
+	 that instead of being passed another AP to be compared to, it is passed sets of
+	 attributes and properties which only virtually comprise another AP.
+	 \retval TRUE if equivalent (regardless of order) to given Attrs and Props, FALSE otherwise.
+*/
 bool PP_AttrProp::isEquivalent(const XML_Char ** attrs, const XML_Char ** props) const
 {
 	UT_uint32 iAttrsCount  = 0;
@@ -1261,11 +1440,11 @@ bool PP_AttrProp::isEquivalent(const XML_Char ** attrs, const XML_Char ** props)
 			return false;
 
 		// ignore property attribute
-		if(0 != UT_strcmp(pValue, PT_PROPS_ATTRIBUTE_NAME))
+		if(0 == UT_strcmp(pValue, PT_PROPS_ATTRIBUTE_NAME))
 			continue;
 
 		// handle revision attribute correctly
-		if(0 != UT_strcmp(pValue, PT_REVISION_ATTRIBUTE_NAME))
+		if(0 == UT_strcmp(pValue, PT_REVISION_ATTRIBUTE_NAME))
 		{
 			// requires special treatment
 			PP_RevisionAttr r1(pValue);
@@ -1295,11 +1474,10 @@ bool PP_AttrProp::isEquivalent(const XML_Char ** attrs, const XML_Char ** props)
 	return true;
 }
 
-/*!
-    This function transfers attributes and properties defined in style into the AP
-
-    bOverwrite indicates what happens if the property/attribute is already present. If false
-    (default) the style definition is ignored; if true the style value overrides the present value
+/*! \fn bool PP_AttrProp::explodeStyle(const PD_Document * pDoc, bool bOverwrite)
+	 \brief This function transfers attributes and properties defined in style into the AP.
+    \param bOverwrite indicates what happens if the property/attribute is already present. If false \
+     (default) the style definition is ignored; if true the style value overrides the present value.
 */
 bool PP_AttrProp::explodeStyle(const PD_Document * pDoc, bool bOverwrite)
 {
@@ -1361,6 +1539,9 @@ bool PP_AttrProp::explodeStyle(const PD_Document * pDoc, bool bOverwrite)
 	return true;
 }
 
+/*! This is a debugging tool which serves to dump in readable form (as UT_DEBUGMSGs)
+	 the contents of [this] AP.
+*/
 void PP_AttrProp::miniDump(const PD_Document * pDoc) const
 {
 #ifdef DEBUG
