@@ -86,6 +86,22 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 {
 	if(m_pDocument->isMarkRevisions())
 	{
+		
+		bool bHdrFtr = false;
+		// if the user selected the whole document for deletion, we will not delete the
+		// first block (we need always a visible block in any document); we make an
+		// exception to this in VDND, because in that case the original block is
+		// guaranteed to come back
+		// 
+		// NB: it is possible that the user might delete all contents in several separate
+		// steps; there is no easy way to protect against that
+		bool bWholeDoc = false;
+		if(!m_pDocument->isVDNDinProgress())
+		{
+			pf_Frag * pLast = getFragments().getLast();
+			bWholeDoc = (dpos1 <= 2 && pLast->getPos() == dpos2);
+		}
+		
 		iRealDeleteCount = 0;
 
 		const XML_Char name[] = "revision";
@@ -135,6 +151,12 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 				{
 					case PTX_Block:
 						iLen = pf_FRAG_STRUX_BLOCK_LENGTH;
+						if(bWholeDoc && dpos1 == 2)
+						{
+							dpos1 += iLen;
+							continue;
+						}
+						
 						break;
 						
 					case PTX_SectionTable:
@@ -155,13 +177,17 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 					case PTX_SectionEndnote:
 					case PTX_SectionFootnote:
 					case PTX_SectionFrame:
+					case PTX_SectionTOC:
 						bHasEndStrux = true;
 						// fall through ...
-					case PTX_Section:
 					case PTX_SectionHdrFtr:
+						bHdrFtr = true;
+						// fall through
+					case PTX_Section:
 				    case PTX_EndFootnote:
 				    case PTX_EndEndnote:
 				    case PTX_EndFrame:
+					case PTX_EndTOC:
 						iLen = pf_FRAG_STRUX_SECTION_LENGTH;
 						break;
 
@@ -227,7 +253,7 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 				{
 					// if this fragment is one of the struxes that is paired with an end-strux, we
 					// need to delete both struxes and everything in between
-					if(bHasEndStrux)
+					if(bHasEndStrux || bHdrFtr)
 					{
 						PT_DocPosition posEnd = dposEnd;
 						for(pf_Frag * pf = pf1->getNext(); pf != NULL; pf = pf->getNext())
@@ -268,6 +294,14 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 									if(eStrux2Type != PTX_EndFrame)
 										continue;
 									break;
+								case PTX_SectionTOC:
+									if(eStrux2Type != PTX_EndTOC)
+										continue;
+									break;
+								case PTX_SectionHdrFtr:
+									if(eStrux2Type != PTX_SectionHdrFtr)
+										continue;
+									break;
 							    default:
 									break;
 							}
@@ -279,11 +313,36 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 						
 						dposEnd = posEnd;
 					}
-					
-					if(!_realDeleteSpan(dpos1, dposEnd, p_AttrProp_Before,bDeleteTableStruxes,
-										bDontGlob))
-						return false;
 
+					if(bHdrFtr)
+					{
+						bHdrFtr = false; // only do this once
+						pf_Frag_Strux_SectionHdrFtr * pfHdr = static_cast<pf_Frag_Strux_SectionHdrFtr *>(pf1);
+
+						const PP_AttrProp * pAP = NULL;
+
+						if(!getAttrProp(pfHdr->getIndexAP(),&pAP) || !pAP)
+							return false;
+
+						const XML_Char * pszHdrId;
+						if(!pAP->getAttribute("id", pszHdrId) || !pszHdrId)
+							return false;
+
+						const XML_Char * pszHdrType;
+						if(!pAP->getAttribute("type", pszHdrType) || !pszHdrType)
+							return false;
+
+						// needs to be in this order because of undo
+						_realDeleteHdrFtrStrux(static_cast<pf_Frag_Strux*>(pf1));
+						_fixHdrFtrReferences(pszHdrType, pszHdrId);
+					}
+					else
+					{
+						if(!_realDeleteSpan(dpos1, dposEnd, p_AttrProp_Before,bDeleteTableStruxes,
+											bDontGlob))
+							return false;
+					}
+					
 					UT_uint32 iDeleteThisStep = dposEnd - dpos1;
 					
 					iRealDeleteCount += iDeleteThisStep;
@@ -309,7 +368,7 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 			{
 				case pf_Frag::PFT_Object:
 				case pf_Frag::PFT_Text:
-					if(! _realChangeSpanFmt(PTC_AddFmt, dpos1, dposEnd, ppRevAttrib,NULL))
+					if(! _realChangeSpanFmt(PTC_AddFmt, dpos1, dposEnd, ppRevAttrib,NULL,true))
 						return false;
 					break;
 
@@ -319,8 +378,36 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 					// it the position immediately after the strux; we
 					// only want the one strux changed, so we pass
 					// identical position in both parameters
-					if(! _realChangeStruxFmt(PTC_AddFmt, dpos1 + iLen, dpos1 + iLen /*2*iLen*/, ppRevAttrib,NULL,eStruxType))
+					if(! _realChangeStruxFmt(PTC_AddFmt, dpos1 + iLen, dpos1 + iLen /*2*iLen*/, ppRevAttrib,NULL,
+											 eStruxType,true))
 						return false;
+
+					if(bHdrFtr)
+					{
+						// even though this is just a notional removal, we still have to
+						// fix the references
+						bHdrFtr = false; // only do this once
+						pf_Frag_Strux_SectionHdrFtr * pfHdr = static_cast<pf_Frag_Strux_SectionHdrFtr *>(pf1);
+
+						const PP_AttrProp * pAP = NULL;
+
+						if(!getAttrProp(pfHdr->getIndexAP(),&pAP) || !pAP)
+							return false;
+
+						const XML_Char * pszHdrId;
+						if(!pAP->getAttribute("id", pszHdrId) || !pszHdrId)
+							return false;
+
+						const XML_Char * pszHdrType;
+						if(!pAP->getAttribute("type", pszHdrType) || !pszHdrType)
+							return false;
+
+						_fixHdrFtrReferences(pszHdrType, pszHdrId, true);
+                        // empty the strux listener chache since the pointers are now
+                        // invalid                                              
+                        pfHdr->clearAllFmtHandles();                            
+					}
+					
 					break;
 
 				default:;
@@ -336,7 +423,117 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 							   bDontGlob);
 }
 
+/*!
+    scan piecetable and remove any references to the hdr/ftr located at pos dpos
+    bNotional indicates that the header has been marked deleted in revison mode, but not
+    physically removed from the document
+*/
+bool pt_PieceTable::_fixHdrFtrReferences(const XML_Char * pszHdrType, const XML_Char * pszHdrId,
+										 bool bNotional /* = false */)
+{
+	UT_return_val_if_fail( pszHdrType && pszHdrId, false );
+	
+	bool bRet = true;
+	const PP_AttrProp * pAP = NULL;
 
+	// look for any doc sections that referrence this header type and id
+	const pf_Frag * pFrag = m_fragments.getFirst();
+	while(pFrag)
+	{
+		if(pFrag->getType() == pf_Frag::PFT_Strux &&
+		   static_cast<const pf_Frag_Strux*>(pFrag)->getStruxType()==PTX_Section)
+		{
+			if(!getAttrProp(pFrag->getIndexAP(),&pAP) || !pAP)
+				continue;
+
+			// check for normal attribute
+			const XML_Char * pszMyHdrId = NULL;
+			if(pAP->getAttribute(pszHdrType, pszMyHdrId) && pszMyHdrId)
+			{
+				if(0 == strcmp(pszMyHdrId, pszHdrId))
+				{
+					const XML_Char* pAttrs [3];
+					pAttrs[0] = pszHdrType;
+					pAttrs[1] = "";
+					pAttrs[2] = NULL;
+
+					bRet &= _fmtChangeStruxWithNotify(PTC_SetFmt, (pf_Frag_Strux*)pFrag,
+													  pAttrs, NULL, false);
+				}
+			}
+
+			// now check for revision attribute ...
+			const XML_Char * pszRevision;
+			if(pAP->getAttribute("revision", pszRevision) && pszRevision)
+			{
+				bool bFound = false;
+				PP_RevisionAttr Revisions(pszRevision);
+
+				for(UT_uint32 i = 0; i < Revisions.getRevisionsCount(); ++i)
+				{
+					const PP_Revision * pRev = Revisions.getNthRevision(i);
+					UT_return_val_if_fail( pRev, false );
+
+					const XML_Char * pszMyHdrId = NULL;
+					if(pRev->getAttribute(pszHdrType, pszMyHdrId) && pszMyHdrId)
+					{
+						if(0 != strcmp(pszHdrId, pszMyHdrId))
+							continue;
+
+						if(!bNotional)
+						{
+							// NB: this is safe, since we own the PP_RevisionAttr object
+							// of local scope which in turn owns this revisions
+							const_cast<PP_Revision*>(pRev)->setAttribute(pszHdrType, "");
+						}
+						else
+						{
+							UT_uint32 iId = m_pDocument->getRevisionId();
+							UT_uint32 iMinId;
+							const PP_Revision * pRev = Revisions.getRevisionWithId(iId, iMinId);
+							if(pRev)
+							{
+								// NB: this is safe, since we own the PP_RevisionAttr object
+								// of local scope which in turn owns this revisions
+								const_cast<PP_Revision*>(pRev)->setAttribute(pszHdrType, "");
+							}
+							else
+							{
+								// we have a section that references this header in
+								// previous revision and has no changes in the current
+								// revision, so we need to add a new revisions in which
+								// the header is not referenced
+								const XML_Char * pAttrs [3] = {pszHdrType, pszHdrId, NULL};
+								Revisions.addRevision(iId, PP_REVISION_FMT_CHANGE, pAttrs, NULL);
+							}
+						}
+
+						Revisions.forceDirty();
+						bFound = true;
+					}
+				}
+				
+				if(bFound)
+				{
+					const XML_Char* pAttrs [3];
+					pAttrs[0] = "revision";
+					pAttrs[1] = Revisions.getXMLstring();
+					pAttrs[2] = NULL;
+
+					bRet &= _fmtChangeStruxWithNotify(PTC_SetFmt, (pf_Frag_Strux*)pFrag,
+													  pAttrs, NULL, false);
+				}
+			}
+								
+								
+		}
+							
+		pFrag = pFrag->getNext();
+	}
+
+
+	return bRet;
+}
 
 bool pt_PieceTable::_deleteSpan(pf_Frag_Text * pft, UT_uint32 fragOffset,
 								PT_BufIndex bi, UT_uint32 length,
@@ -574,15 +771,24 @@ bool pt_PieceTable::_tweakDeleteSpanOnce(PT_DocPosition & dpos1,
 #endif
 	case PTX_SectionHdrFtr:
 		// if the previous container is a Header/Footersection, then pf_First
-		// must be the first block in the section.
+		// must be the first block or the first Table in the section.
 		UT_return_val_if_fail ((pf_First->getPrev() == pfsContainer),false);
 		UT_return_val_if_fail ((pf_First->getType() == pf_Frag::PFT_Strux),false);
-		UT_return_val_if_fail (((static_cast<pf_Frag_Strux *>(pf_First))->getStruxType() == PTX_Block),false);
+		UT_return_val_if_fail ((((static_cast<pf_Frag_Strux *>(pf_First))->getStruxType() == PTX_Block) || (static_cast<pf_Frag_Strux *>(pf_First))->getStruxType() == PTX_SectionTable),false);
+
+		//
+		// This allows us to delete the first Table in a section
+		//
+		if(static_cast<pf_Frag_Strux *>(pf_First)->getStruxType() == PTX_SectionTable)
+		{
+		     return true;
+		}
 		// since, we cannot delete the first block in a section, we
 		// secretly translate this into a request to delete the section;
 		// the block we have will then be slurped into the previous
 		// section.
 		dpos1 -= pfsContainer->getLength();
+		
 		return true;
 
 	case PTX_SectionTable:
@@ -1010,6 +1216,14 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition & origPos1,
 	if(pfsContainer->getStruxType() == PTX_SectionFrame)
 	{
 		bPrevWasFrame = true;
+	        if(pf_First->getType() == pf_Frag::PFT_Strux)
+		{
+		     pf_Frag_Strux * pfs = static_cast<pf_Frag_Strux *>(pf_First);
+		     if(pfs->getStruxType() == PTX_SectionTable)
+		     {
+		          bPrevWasFrame = false;
+		     }
+		}
 	}
 	if(pfsContainer->getStruxType() == PTX_SectionCell)
 	{
@@ -1090,9 +1304,20 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition & origPos1,
 // 
 					if(!isFootnote(pfs) && !isEndFootnote(pfs))
 					{ 
-						UT_DEBUGMSG(("Delete Block strux \n"));
+						UT_DEBUGMSG(("Delete Block strux dpos1 %d \n",dpos1));
 						bResult = _deleteStruxWithNotify(dpos1,pfs,
-												  &pfNewEnd,&fragOffsetNewEnd);
+										 &pfNewEnd,&fragOffsetNewEnd);
+						
+						if(!bResult) // can't delete this block strux
+						             // but can delete the rest of the content
+						{
+						  UT_DEBUGMSG(("dpos1 = %d \n",dpos1));
+						  pfNewEnd = pfs->getNext();
+						  dpos1 += pfs->getLength();
+						  pfsContainer = pfs;
+						  fragOffsetNewEnd = 0;
+						}
+						// UT_return_val_if_fail(bResult,false);
 						bPrevWasCell = false;
 						bPrevWasFrame = false;
 						bPrevWasEndTable = false;
@@ -1353,7 +1578,7 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition & origPos1,
 			UT_return_val_if_fail (bResult, false);
 		}
 		break;
-		// the bookmark and hyperlink objects require special treatment; since
+		// the bookmark and hyperlink objects require specxial treatment; since
 		// they come in pairs, we have to ensure that both are deleted together
 		// so we have to find the other part of the pair, delete it, adjust the
 		// positional variables and the delete the one we were originally asked
