@@ -22,6 +22,7 @@
 
 #include "ap_Win32Frame.h"
 #include "ut_Win32Timer.h"
+#include "ut_Win32OS.h"
 #include "gr_Win32Graphics.h"
 #include "ap_Win32TopRuler.h"
 #include "ap_Win32LeftRuler.h"
@@ -39,6 +40,16 @@
 #define SPI_GETWHEELSCROLLLINES   104
 #endif
 
+// I really hate doing this, but this constant is only defined with _WIN32_WINNT >=
+// 0x0501, i.e., winXP, and I do not want to enable the lot
+#ifndef WM_UNICHAR
+#define WM_UNICHAR 0x109
+#endif
+
+#ifndef UNICODE_NOCHAR
+#define UNICODE_NOCHAR 0xFFFF
+#endif
+
 #define GWL(hwnd)		reinterpret_cast<AP_Win32Frame *>(GetWindowLong((hwnd), GWL_USERDATA))
 #define SWL(hwnd, f)	reinterpret_cast<AP_Win32Frame *>(SetWindowLong((hwnd), GWL_USERDATA,(LONG)(f)))
 
@@ -101,7 +112,7 @@ HWND AP_Win32FrameImpl::_createDocumentWindow(XAP_Frame *pFrame, HWND hwndParent
 	// the 'container' will in turn contain the document window, the
 	// rulers, and the variousscroll bars and other dead space.
 
-	m_hwndContainer = CreateWindowEx(WS_EX_CLIENTEDGE, s_ContainerWndClassName, NULL,
+	m_hwndContainer = UT_CreateWindowEx(WS_EX_CLIENTEDGE, s_ContainerWndClassName, NULL,
 									 WS_CHILD | WS_VISIBLE,
 									 iLeft, iTop, iWidth, iHeight,
 									 hwndParent, NULL, static_cast<XAP_Win32App *>(XAP_App::getApp())->getInstance(), NULL);
@@ -117,7 +128,7 @@ HWND AP_Win32FrameImpl::_createDocumentWindow(XAP_Frame *pFrame, HWND hwndParent
 	const int cyHScroll = GetSystemMetrics(SM_CYHSCROLL);
 	const int cxVScroll = GetSystemMetrics(SM_CXVSCROLL);
 
-	m_hWndHScroll = CreateWindowEx(0, "ScrollBar", 0, WS_CHILD | WS_VISIBLE | SBS_HORZ,
+	m_hWndHScroll = UT_CreateWindowEx(0, "ScrollBar", 0, WS_CHILD | WS_VISIBLE | SBS_HORZ,
 									0, r.bottom - cyHScroll,
 									r.right - cxVScroll, cyHScroll,
 									m_hwndContainer,
@@ -127,7 +138,7 @@ HWND AP_Win32FrameImpl::_createDocumentWindow(XAP_Frame *pFrame, HWND hwndParent
 	//SWL(m_hWndHScroll, this);
 	SWL(m_hWndHScroll, pFrame);
 
-	m_hWndVScroll = CreateWindowEx(0, "ScrollBar", 0, WS_CHILD | WS_VISIBLE | SBS_VERT,
+	m_hWndVScroll = UT_CreateWindowEx(0, "ScrollBar", 0, WS_CHILD | WS_VISIBLE | SBS_VERT,
 									r.right - cxVScroll, 0,
 									cxVScroll, r.bottom - cyHScroll,
 									m_hwndContainer,
@@ -143,7 +154,7 @@ HWND AP_Win32FrameImpl::_createDocumentWindow(XAP_Frame *pFrame, HWND hwndParent
 #  define XX_StyleBits          (SBS_SIZEGRIP)
 #endif
 
-	m_hWndGripperHack = CreateWindowEx(0,"ScrollBar", 0,
+	m_hWndGripperHack = UT_CreateWindowEx(0,"ScrollBar", 0,
 										WS_CHILD | WS_VISIBLE | XX_StyleBits,
 										r.right-cxVScroll, r.bottom-cyHScroll, cxVScroll, cyHScroll,
 										m_hwndContainer, NULL, static_cast<XAP_Win32App *>(XAP_App::getApp())->getInstance(), NULL);
@@ -176,12 +187,14 @@ HWND AP_Win32FrameImpl::_createDocumentWindow(XAP_Frame *pFrame, HWND hwndParent
 	}
 
 	// create a child window for us.
-	m_hwndDocument = CreateWindowEx(0, s_DocumentWndClassName, NULL,
-									WS_CHILD | WS_VISIBLE,
-									xLeftRulerWidth, yTopRulerHeight,
-									r.right - xLeftRulerWidth - cxVScroll,
-									r.bottom - yTopRulerHeight - cyHScroll,
-									m_hwndContainer, NULL, static_cast<XAP_Win32App *>(XAP_App::getApp())->getInstance(), NULL);
+	m_hwndDocument = UT_CreateWindowEx(0, s_DocumentWndClassName, NULL,
+									   WS_CHILD | WS_VISIBLE,
+									   xLeftRulerWidth, yTopRulerHeight,
+									   r.right - xLeftRulerWidth - cxVScroll,
+									   r.bottom - yTopRulerHeight - cyHScroll,
+									   m_hwndContainer, NULL,
+									   static_cast<XAP_Win32App *>(XAP_App::getApp())->getInstance(), NULL);
+	
 	UT_return_val_if_fail (m_hwndDocument, 0);
 	// WARNING!!! many places expact an XAP_Frame or descendant!!!
 	//SWL(m_hwndDocument, this);
@@ -576,7 +589,6 @@ bool AP_Win32FrameImpl::_RegisterClass(XAP_Win32App * app)
 	if (!XAP_Win32FrameImpl::_RegisterClass(app))
 		return false;
 
-	WNDCLASSEX  wndclass;
 	ATOM a;
 
 	// register class for the container window (this will contain the document
@@ -584,41 +596,19 @@ bool AP_Win32FrameImpl::_RegisterClass(XAP_Win32App * app)
 
 	_snprintf(s_ContainerWndClassName, MAXCNTWNDCLSNMSIZE, "%sContainer", app->getApplicationName());
 
-	memset(&wndclass, 0, sizeof(wndclass));
-	wndclass.cbSize        = sizeof(wndclass);
-	wndclass.style         = CS_DBLCLKS | CS_OWNDC;
-	wndclass.lpfnWndProc   = AP_Win32FrameImpl::_ContainerWndProc;
-	wndclass.cbClsExtra    = 0;
-	wndclass.cbWndExtra    = 0;
-	wndclass.hInstance     = app->getInstance();
-	wndclass.hIcon         = NULL;
-	wndclass.hCursor       = LoadCursor(NULL, IDC_ARROW);
-	wndclass.hbrBackground = NULL;
-	wndclass.lpszMenuName  = NULL;
-	wndclass.lpszClassName = s_ContainerWndClassName;
-	wndclass.hIconSm       = NULL;
-
-	a = RegisterClassEx(&wndclass);
+	a = UT_RegisterClassEx(CS_DBLCLKS | CS_OWNDC, AP_Win32FrameImpl::_ContainerWndProc, app->getInstance(),
+						   NULL, LoadCursor(NULL, IDC_ARROW), NULL, NULL,
+						   NULL, s_ContainerWndClassName);
+	
 	UT_return_val_if_fail(a, false);
 
 	// register class for the actual document window
 	_snprintf(s_DocumentWndClassName, MAXDOCWNDCLSNMSIZE, "%sDocument", app->getApplicationName());
 
-	memset(&wndclass, 0, sizeof(wndclass));
-	wndclass.cbSize        = sizeof(wndclass);
-	wndclass.style         = CS_DBLCLKS | CS_OWNDC;
-	wndclass.lpfnWndProc   = AP_Win32FrameImpl::_DocumentWndProc;
-	wndclass.cbClsExtra    = 0;
-	wndclass.cbWndExtra    = 0;
-	wndclass.hInstance     = app->getInstance();
-	wndclass.hIcon         = NULL;
-	wndclass.hCursor       = NULL;
-	wndclass.hbrBackground = NULL;
-	wndclass.lpszMenuName  = NULL;
-	wndclass.lpszClassName = s_DocumentWndClassName;
-	wndclass.hIconSm       = NULL;
+	a = UT_RegisterClassEx(CS_DBLCLKS | CS_OWNDC, AP_Win32FrameImpl::_DocumentWndProc, app->getInstance(),
+						   NULL, NULL, NULL, NULL,
+						   NULL, s_DocumentWndClassName);
 
-	a = RegisterClassEx(&wndclass);
 	UT_return_val_if_fail(a, false);
 
 	if (!AP_Win32TopRuler::RegisterClass(app) ||
@@ -817,11 +807,11 @@ LRESULT CALLBACK AP_Win32FrameImpl::_ContainerWndProc(HWND hwnd, UINT iMsg, WPAR
 
 	if (!f)
 	{
-		return DefWindowProc(hwnd, iMsg, wParam, lParam);
+		return UT_DefWindowProc(hwnd, iMsg, wParam, lParam);
 	}
 
 	AP_Win32FrameImpl *fImpl = f->getAPWin32FrameImpl();
-	UT_return_val_if_fail(fImpl, DefWindowProc(hwnd, iMsg, wParam, lParam));
+	UT_return_val_if_fail(fImpl, UT_DefWindowProc(hwnd, iMsg, wParam, lParam));
 
 	AV_View* pView = f->getCurrentView();
 	//UT_ASSERT(pView);		/* only fatal for some messages */
@@ -950,7 +940,7 @@ LRESULT CALLBACK AP_Win32FrameImpl::_ContainerWndProc(HWND hwnd, UINT iMsg, WPAR
 			SendMessage(fImpl->m_hwndTopRuler,WM_SYSCOLORCHANGE,0,0);
 			SendMessage(fImpl->m_hwndLeftRuler,WM_SYSCOLORCHANGE,0,0);
 			SendMessage(fImpl->m_hwndDocument,WM_SYSCOLORCHANGE,0,0);
-			return DefWindowProc(hwnd, iMsg, wParam, lParam);
+			return UT_DefWindowProc(hwnd, iMsg, wParam, lParam);
 		}
 
 	 	case WM_MOUSEWHEEL:
@@ -1011,8 +1001,10 @@ LRESULT CALLBACK AP_Win32FrameImpl::_ContainerWndProc(HWND hwnd, UINT iMsg, WPAR
 		}
 
 		default:
-			return DefWindowProc(hwnd, iMsg, wParam, lParam);
+			return UT_DefWindowProc(hwnd, iMsg, wParam, lParam);
 	}
+
+	return 0; // to silence the compiler
 }
 
 LRESULT CALLBACK AP_Win32FrameImpl::_DocumentWndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
@@ -1021,11 +1013,11 @@ LRESULT CALLBACK AP_Win32FrameImpl::_DocumentWndProc(HWND hwnd, UINT iMsg, WPARA
 
 	if (!f)
 	{
-		return DefWindowProc(hwnd, iMsg, wParam, lParam);
+		return UT_DefWindowProc(hwnd, iMsg, wParam, lParam);
 	}
 
 	AP_Win32FrameImpl *fImpl = static_cast<AP_Win32FrameImpl *>(f->getFrameImpl());
-	UT_return_val_if_fail(fImpl, DefWindowProc(hwnd, iMsg, wParam, lParam));
+	UT_return_val_if_fail(fImpl, UT_DefWindowProc(hwnd, iMsg, wParam, lParam));
 
 	AV_View* 		pView = f->getCurrentView();
 	EV_Win32Mouse*	pMouse = (EV_Win32Mouse *) fImpl->m_pMouse;
@@ -1126,7 +1118,7 @@ LRESULT CALLBACK AP_Win32FrameImpl::_DocumentWndProc(HWND hwnd, UINT iMsg, WPARA
 
 	    
 	     	if (pWin32Keyboard->onKeyDown(pView,hwnd,iMsg,wParam,lParam))
-    			return DefWindowProc(hwnd,iMsg,wParam,lParam);
+    			return UT_DefWindowProc(hwnd,iMsg,wParam,lParam);
 	 		else
  				return false;	    				 	     			
 		
@@ -1194,7 +1186,7 @@ LRESULT CALLBACK AP_Win32FrameImpl::_DocumentWndProc(HWND hwnd, UINT iMsg, WPARA
 
 	    
 		     	if (pWin32Keyboard->onKeyDown(pView,hwnd,iMsg,wParam,lParam))
-     				return DefWindowProc(hwnd,iMsg,wParam,lParam);
+     				return UT_DefWindowProc(hwnd,iMsg,wParam,lParam);
 	 		else
  				return false;	    				 	     			
     		
@@ -1204,7 +1196,7 @@ LRESULT CALLBACK AP_Win32FrameImpl::_DocumentWndProc(HWND hwnd, UINT iMsg, WPARA
 		{
 			UT_DEBUGMSG(("WM_SYSCHAR %d  - %d\n",wParam, lParam));
 			ev_Win32Keyboard *pWin32Keyboard = static_cast<ev_Win32Keyboard *>(fImpl->m_pKeyboard);	    
-			return DefWindowProc(hwnd,iMsg,wParam,lParam);
+			return UT_DefWindowProc(hwnd,iMsg,wParam,lParam);
 		}
 
 		case WM_CHAR:
@@ -1213,7 +1205,21 @@ LRESULT CALLBACK AP_Win32FrameImpl::_DocumentWndProc(HWND hwnd, UINT iMsg, WPARA
 			ev_Win32Keyboard *pWin32Keyboard = static_cast<ev_Win32Keyboard *>(fImpl->m_pKeyboard);
 	    
 			pWin32Keyboard->onChar(pView,hwnd,iMsg,wParam,lParam);		
-			return DefWindowProc(hwnd,iMsg,wParam,lParam);
+			return UT_DefWindowProc(hwnd,iMsg,wParam,lParam);
+		}
+
+		case WM_UNICHAR:
+		{
+			UT_DEBUGMSG(("WM_CHAR %d  - %d\n",wParam, lParam));
+
+			// UNICODE_NOCHAR is sent to test if we can handle this message
+			if(wParam == UNICODE_NOCHAR)
+				return 1;
+			
+			ev_Win32Keyboard *pWin32Keyboard = static_cast<ev_Win32Keyboard *>(fImpl->m_pKeyboard);
+	    
+			pWin32Keyboard->onUniChar(pView,hwnd,iMsg,wParam,lParam);		
+			return UT_DefWindowProc(hwnd,iMsg,wParam,lParam);
 		}
 
 		case WM_IME_CHAR:
@@ -1221,7 +1227,7 @@ LRESULT CALLBACK AP_Win32FrameImpl::_DocumentWndProc(HWND hwnd, UINT iMsg, WPARA
 			ev_Win32Keyboard *pWin32Keyboard = static_cast<ev_Win32Keyboard *>(fImpl->m_pKeyboard);
 			if (pWin32Keyboard->onIMEChar(pView,hwnd,iMsg,wParam,lParam))
 				return 0;
-			return DefWindowProc(hwnd,iMsg,wParam,lParam);
+			return UT_DefWindowProc(hwnd,iMsg,wParam,lParam);
 		}
 
 		case WM_MOUSEMOVE:
@@ -1343,7 +1349,7 @@ LRESULT CALLBACK AP_Win32FrameImpl::_DocumentWndProc(HWND hwnd, UINT iMsg, WPARA
 #endif
 	} /* switch (iMsg) */
 
-	return DefWindowProc(hwnd, iMsg, wParam, lParam);
+	return UT_DefWindowProc(hwnd, iMsg, wParam, lParam);
 }
 
 UT_RGBColor AP_Win32FrameImpl::getColorSelBackground () const
@@ -1362,7 +1368,7 @@ UT_RGBColor AP_Win32FrameImpl::getColorSelBackground () const
 	return UT_RGBColor( red, green, blue );
 }
 
-GR_Win32Graphics *AP_Win32FrameImpl::_createDocWnd_GR_Graphics(void)
+GR_Win32Graphics *AP_Win32FrameImpl::createDocWndGraphics(void)
 {
 	GR_Win32AllocInfo ai(GetDC(getHwndDocument()), getHwndDocument(), XAP_App::getApp());
 	return (GR_Win32Graphics *)XAP_App::getApp()->newGraphics(ai);
