@@ -17,6 +17,10 @@
  * 02111-1307, USA.
  */
 
+#include "ap_Features.h"
+#ifdef APF_MENU_LAYOUTS_MAIN_MENU
+#  include APF_MENU_LAYOUTS_MAIN_MENU
+#else
 
 /*****************************************************************
 ******************************************************************
@@ -31,14 +35,37 @@ BeginLayout(Main,0)
 		MenuItem(AP_MENU_ID_FILE_NEW)
 		MenuItem(AP_MENU_ID_FILE_NEW_USING_TEMPLATE)
 		MenuItem(AP_MENU_ID_FILE_OPEN)
+	
+#if !XP_SIMPLE_MENU	
+		MenuItem(AP_MENU_ID_FILE_IMPORTSTYLES)
+#endif	
+
+		Separator()
+
 		MenuItem(AP_MENU_ID_FILE_SAVE)
 		MenuItem(AP_MENU_ID_FILE_SAVEAS)
 //		MenuItem(AP_MENU_ID_FILE_SAVE_TEMPLATE)
-		Separator()
-		MenuItem(AP_MENU_ID_FILE_IMPORT)
+//		MenuItem(AP_MENU_ID_FILE_IMPORT)
 		MenuItem(AP_MENU_ID_FILE_EXPORT)
 		MenuItem(AP_MENU_ID_FILE_REVERT)
 
+#if !XP_SIMPLE_MENU
+		Separator()
+		MenuItem(AP_MENU_ID_FILE_PAGESETUP)
+#ifdef XP_UNIX_TARGET_GTK
+		MenuItem(AP_MENU_ID_FILE_PRINT_PREVIEW)
+#endif
+		MenuItem(AP_MENU_ID_FILE_PRINT)
+		Separator()
+		MenuItem(AP_MENU_ID_FILE_PROPERTIES)
+		Separator()
+#ifdef XP_UNIX_TARGET_GTK
+		// GNOME HIG style recent files 
+		MenuItem(AP_MENU_ID_FILE_RECENT_1)
+		MenuItem(AP_MENU_ID_FILE_RECENT_2)
+		MenuItem(AP_MENU_ID_FILE_RECENT_3)
+		MenuItem(AP_MENU_ID_FILE_RECENT_4)
+#else
 		BeginSubMenu(AP_MENU_ID_FILE_RECENT)
 			MenuItem(AP_MENU_ID_FILE_RECENT_1)
 			MenuItem(AP_MENU_ID_FILE_RECENT_2)
@@ -50,16 +77,12 @@ BeginLayout(Main,0)
 			MenuItem(AP_MENU_ID_FILE_RECENT_8)
 			MenuItem(AP_MENU_ID_FILE_RECENT_9)
 		EndSubMenu()
-
-		Separator()
-		MenuItem(AP_MENU_ID_FILE_PROPERTIES)
-		MenuItem(AP_MENU_ID_FILE_PAGESETUP)
-#if defined(HAVE_GNOME)
-		MenuItem(AP_MENU_ID_FILE_PRINT_PREVIEW)
 #endif
-		MenuItem(AP_MENU_ID_FILE_PRINT)
+#endif
 		Separator()
+#if  !XP_SIMPLE_MENU
 		MenuItem(AP_MENU_ID_FILE_CLOSE)
+#endif
 		MenuItem(AP_MENU_ID_FILE_EXIT)
 	EndSubMenu()
 
@@ -70,20 +93,32 @@ BeginLayout(Main,0)
 		MenuItem(AP_MENU_ID_EDIT_CUT)
 		MenuItem(AP_MENU_ID_EDIT_COPY)
 		MenuItem(AP_MENU_ID_EDIT_PASTE)
+#if !XP_SIMPLE_MENU	
 		MenuItem(AP_MENU_ID_EDIT_PASTE_SPECIAL)
+#endif	
 		Separator()
+#if !XP_SIMPLE_MENU	
 		MenuItem(AP_MENU_ID_EDIT_CLEAR)
+#endif	
 		MenuItem(AP_MENU_ID_EDIT_SELECTALL)
 		Separator()
+#if !XP_SIMPLE_MENU	
+		MenuItem(AP_MENU_ID_EDIT_REMOVEHEADER)
+		MenuItem(AP_MENU_ID_EDIT_REMOVEFOOTER)
+		Separator()
+#endif	
 		MenuItem(AP_MENU_ID_EDIT_FIND)
 		MenuItem(AP_MENU_ID_EDIT_REPLACE)
+#if !XP_SIMPLE_MENU	
 		MenuItem(AP_MENU_ID_EDIT_GOTO)
+#endif	
 #if !(XAP_PREFSMENU_UNDER_TOOLS)
 		Separator()
 		MenuItem(AP_MENU_ID_TOOLS_OPTIONS)
 #endif
 	EndSubMenu()
 
+#if !XP_SIMPLE_MENU
 	BeginSubMenu(AP_MENU_ID_VIEW)
 		MenuItem(AP_MENU_ID_VIEW_NORMAL)
 		MenuItem(AP_MENU_ID_VIEW_WEB)
@@ -121,9 +156,9 @@ BeginLayout(Main,0)
 			MenuItem(AP_MENU_ID_VIEW_ZOOM_75)
 			MenuItem(AP_MENU_ID_VIEW_ZOOM_50)
 		EndSubMenu()
-
 	EndSubMenu()
-
+#endif /*  XP_SIMPLE_MENU */
+	
 	BeginSubMenu(AP_MENU_ID_INSERT)
 		MenuItem(AP_MENU_ID_INSERT_BREAK)
 		MenuItem(AP_MENU_ID_INSERT_PAGENO)
@@ -133,70 +168,8 @@ BeginLayout(Main,0)
 		MenuItem(AP_MENU_ID_INSERT_MAILMERGE)
 		MenuItem(AP_MENU_ID_INSERT_SYMBOL)
 
-		/* the autotext submenus */
-
-		BeginSubMenu(AP_MENU_ID_INSERT_AUTOTEXT)
-
-			BeginSubMenu(AP_MENU_ID_AUTOTEXT_ATTN)
-				MenuItem(AP_MENU_ID_AUTOTEXT_ATTN_1)
-				MenuItem(AP_MENU_ID_AUTOTEXT_ATTN_2)
-			EndSubMenu()
-
-			BeginSubMenu(AP_MENU_ID_AUTOTEXT_CLOSING)
-				MenuItem(AP_MENU_ID_AUTOTEXT_CLOSING_1)
-				MenuItem(AP_MENU_ID_AUTOTEXT_CLOSING_2)
-				MenuItem(AP_MENU_ID_AUTOTEXT_CLOSING_3)
-				MenuItem(AP_MENU_ID_AUTOTEXT_CLOSING_4)
-				MenuItem(AP_MENU_ID_AUTOTEXT_CLOSING_5)
-				MenuItem(AP_MENU_ID_AUTOTEXT_CLOSING_6)
-				MenuItem(AP_MENU_ID_AUTOTEXT_CLOSING_7)
-				MenuItem(AP_MENU_ID_AUTOTEXT_CLOSING_8)
-				MenuItem(AP_MENU_ID_AUTOTEXT_CLOSING_9)
-				MenuItem(AP_MENU_ID_AUTOTEXT_CLOSING_10)
-				MenuItem(AP_MENU_ID_AUTOTEXT_CLOSING_11)
-				MenuItem(AP_MENU_ID_AUTOTEXT_CLOSING_12)
-			EndSubMenu()
-
-			BeginSubMenu(AP_MENU_ID_AUTOTEXT_MAIL)
-				MenuItem(AP_MENU_ID_AUTOTEXT_MAIL_1)
-				MenuItem(AP_MENU_ID_AUTOTEXT_MAIL_2)
-				MenuItem(AP_MENU_ID_AUTOTEXT_MAIL_3)
-				MenuItem(AP_MENU_ID_AUTOTEXT_MAIL_4)
-				MenuItem(AP_MENU_ID_AUTOTEXT_MAIL_5)
-				MenuItem(AP_MENU_ID_AUTOTEXT_MAIL_6)
-				MenuItem(AP_MENU_ID_AUTOTEXT_MAIL_7)
-				MenuItem(AP_MENU_ID_AUTOTEXT_MAIL_8)
-			EndSubMenu()
-
-			BeginSubMenu(AP_MENU_ID_AUTOTEXT_REFERENCE)
-				MenuItem(AP_MENU_ID_AUTOTEXT_REFERENCE_1)
-				MenuItem(AP_MENU_ID_AUTOTEXT_REFERENCE_2)
-				MenuItem(AP_MENU_ID_AUTOTEXT_REFERENCE_3)
-			EndSubMenu()
-
-			BeginSubMenu(AP_MENU_ID_AUTOTEXT_SALUTATION)
-				MenuItem(AP_MENU_ID_AUTOTEXT_SALUTATION_1)
-				MenuItem(AP_MENU_ID_AUTOTEXT_SALUTATION_2)
-				MenuItem(AP_MENU_ID_AUTOTEXT_SALUTATION_3)
-				MenuItem(AP_MENU_ID_AUTOTEXT_SALUTATION_4)
-			EndSubMenu()
-
-			BeginSubMenu(AP_MENU_ID_AUTOTEXT_EMAIL)
-				MenuItem(AP_MENU_ID_AUTOTEXT_EMAIL_1)
-				MenuItem(AP_MENU_ID_AUTOTEXT_EMAIL_2)
-				MenuItem(AP_MENU_ID_AUTOTEXT_EMAIL_3)
-				MenuItem(AP_MENU_ID_AUTOTEXT_EMAIL_4)
-				MenuItem(AP_MENU_ID_AUTOTEXT_EMAIL_5)
-				MenuItem(AP_MENU_ID_AUTOTEXT_EMAIL_6)
-			EndSubMenu()
-
-			BeginSubMenu(AP_MENU_ID_AUTOTEXT_SUBJECT)
-				MenuItem(AP_MENU_ID_AUTOTEXT_SUBJECT_1)
-			EndSubMenu()
-
-		EndSubMenu()
-
 		Separator()
+
 		MenuItem(AP_MENU_ID_INSERT_HEADER)
 		MenuItem(AP_MENU_ID_INSERT_FOOTER)
 		MenuItem(AP_MENU_ID_INSERT_FILE)
@@ -227,7 +200,6 @@ BeginLayout(Main,0)
 		MenuItem(AP_MENU_ID_FMT_FONT)
 		MenuItem(AP_MENU_ID_FMT_PARAGRAPH)
 		MenuItem(AP_MENU_ID_FMT_BULLETS)
-		MenuItem(AP_MENU_ID_FMT_DOCUMENT)
 		MenuItem(AP_MENU_ID_FMT_FRAME)
         MenuItem(AP_MENU_ID_FMT_IMAGE)
 #if 0 // someone code and turn this back on
@@ -235,7 +207,9 @@ BeginLayout(Main,0)
 #endif
 		Separator()
 		MenuItem(AP_MENU_ID_FMT_COLUMNS)
+#if !XP_SIMPLE_MENU	
 		MenuItem(AP_MENU_ID_FMT_TABS)
+#endif	
 		MenuItem(AP_MENU_ID_FMT_HDRFTR)
 		MenuItem(AP_MENU_ID_FMT_FOOTNOTES)
 		MenuItem(AP_MENU_ID_FMT_TABLEOFCONTENTS)
@@ -262,37 +236,46 @@ BeginLayout(Main,0)
 			MenuItem(AP_MENU_ID_FMT_SUBSCRIPT)
 		EndSubMenu()
 
-		BeginSubMenu(AP_MENU_ID_FMT_STYLE)
-			MenuItem(AP_MENU_ID_FMT_STYLE_DEFINE)
-			MenuItem(AP_MENU_ID_FMT_IMPORTSTYLES)
-	        MenuItem(AP_MENU_ID_FMT_STYLIST)
-		EndSubMenu()
-		Separator()
-		MenuItem(AP_MENU_ID_EDIT_REMOVEHEADER)
-		MenuItem(AP_MENU_ID_EDIT_REMOVEFOOTER)
-
 		BeginSubMenu(AP_MENU_ID_FMT_BACKGROUND)
-		    MenuItem(AP_MENU_ID_FMT_BACKGROUND_PAGE_IMAGE)
-		    MenuItem(AP_MENU_ID_FMT_BACKGROUND_PAGE_COLOR)
-	    EndSubMenu()
+			MenuItem(AP_MENU_ID_FMT_BACKGROUND_PAGE_IMAGE)
+			MenuItem(AP_MENU_ID_FMT_BACKGROUND_PAGE_COLOR)
+		EndSubMenu()
 
 	    BeginSubMenu(AP_MENU_ID_FMT_DIRECTION)
-	        MenuItem(AP_MENU_ID_FMT_DIRECTION_DD_RTL)
+			MenuItem(AP_MENU_ID_FMT_DIRECTION_DD_RTL)
         	MenuItem(AP_MENU_ID_FMT_DIRECTION_DO_LTR)
         	MenuItem(AP_MENU_ID_FMT_DIRECTION_DO_RTL)
     	EndSubMenu()
+
+#if XP_SIMPLE_MENU
+	BeginSubMenu(AP_MENU_ID_VIEW_ZOOM_MENU)
+		MenuItem(AP_MENU_ID_VIEW_ZOOM)
+		MenuItem(AP_MENU_ID_VIEW_ZOOM_WIDTH)
+		MenuItem(AP_MENU_ID_VIEW_ZOOM_WHOLE)
+		MenuItem(AP_MENU_ID_VIEW_ZOOM_200)
+		MenuItem(AP_MENU_ID_VIEW_ZOOM_100)
+		MenuItem(AP_MENU_ID_VIEW_ZOOM_75)
+		MenuItem(AP_MENU_ID_VIEW_ZOOM_50)
+	EndSubMenu()
+#else	
+
+		Separator()
+
+		MenuItem(AP_MENU_ID_FMT_STYLE_DEFINE)
+	
+
+#endif
+	
 	EndSubMenu()
 
 	BeginSubMenu(AP_MENU_ID_TOOLS)
 
-		BeginSubMenu(AP_MENU_ID_TOOLS_SPELLING)
-			MenuItem(AP_MENU_ID_TOOLS_SPELL)
-			MenuItem(AP_MENU_ID_TOOLS_AUTOSPELL)
-			MenuItem(AP_MENU_ID_TOOLS_SPELLPREFS)
-		EndSubMenu()
-
+		MenuItem(AP_MENU_ID_TOOLS_SPELL)
 		MenuItem(AP_MENU_ID_FMT_LANGUAGE)
 		MenuItem(AP_MENU_ID_TOOLS_WORDCOUNT)
+#if !XP_SIMPLE_MENU	
+        MenuItem(AP_MENU_ID_FMT_STYLIST)
+#endif	
 
 		Separator()
 
@@ -300,10 +283,16 @@ BeginLayout(Main,0)
 	        MenuItem(AP_MENU_ID_TOOLS_HISTORY_SHOW)
 	        MenuItem(AP_MENU_ID_TOOLS_REVISIONS_COMPARE_DOCUMENTS)
 		    MenuItem(AP_MENU_ID_TOOLS_REVISIONS_AUTO)
+#if !XP_SIMPLE_MENU	
+		    MenuItem(AP_MENU_ID_TOOLS_HISTORY_PURGE)
+#endif	
 		EndSubMenu()
 	
 		BeginSubMenu(AP_MENU_ID_TOOLS_REVISIONS)
 			MenuItem(AP_MENU_ID_TOOLS_REVISIONS_MARK)
+#if !XP_SIMPLE_MENU	
+			MenuItem(AP_MENU_ID_TOOLS_REVISIONS_NEW_REVISION)
+#endif	
 		    Separator()
 	        MenuItem(AP_MENU_ID_TOOLS_REVISIONS_SHOW)
 			MenuItem(AP_MENU_ID_TOOLS_REVISIONS_SHOW_AFTER)
@@ -316,6 +305,9 @@ BeginLayout(Main,0)
 	        Separator()
 			MenuItem(AP_MENU_ID_TOOLS_REVISIONS_ACCEPT_REVISION)
 			MenuItem(AP_MENU_ID_TOOLS_REVISIONS_REJECT_REVISION)
+#if !XP_SIMPLE_MENU	
+			MenuItem(AP_MENU_ID_TOOLS_REVISIONS_PURGE)
+#endif	
 #ifdef DEBUG
 	        Separator()
 		    MenuItem(AP_MENU_ID_TOOLS_REVISIONS_MERGE_DOCUMENTS)
@@ -324,6 +316,7 @@ BeginLayout(Main,0)
 	
 	    Separator()
 
+#if !XP_SIMPLE_MENU
 		MenuItem(AP_MENU_ID_TOOLS_PLUGINS)
 		MenuItem(AP_MENU_ID_TOOLS_SCRIPTS)
 		MenuItem(AP_MENU_ID_TOOLS_MAILMERGE)
@@ -334,6 +327,7 @@ BeginLayout(Main,0)
 		Separator()
 		MenuItem(AP_MENU_ID_TOOLS_OPTIONS)
 #endif
+#endif	
 	EndSubMenu()
 
 	BeginSubMenu(AP_MENU_ID_TABLE)
@@ -343,10 +337,11 @@ BeginLayout(Main,0)
 			MenuItem(AP_MENU_ID_TABLE_INSERT_COLUMNS_BEFORE)
 			MenuItem(AP_MENU_ID_TABLE_INSERT_COLUMNS_AFTER)
 			MenuItem(AP_MENU_ID_TABLE_INSERT_ROWS_BEFORE)
+			MenuItem(AP_MENU_ID_TABLE_INSERT_ROWS_AFTER)
 			MenuItem(AP_MENU_ID_TABLE_INSERT_SUMCOLS)
 			MenuItem(AP_MENU_ID_TABLE_INSERT_SUMROWS)
 #if 0
-// Not for 2.2
+// Not for 2.4
 			MenuItem(AP_MENU_ID_TABLE_INSERT_CELLS)
 #endif
 		EndSubMenu()
@@ -356,7 +351,7 @@ BeginLayout(Main,0)
 			MenuItem(AP_MENU_ID_TABLE_DELETE_COLUMNS)
 			MenuItem(AP_MENU_ID_TABLE_DELETE_ROWS)
 #if 0
-// Not for 2.2
+// Not for 2.4
 			MenuItem(AP_MENU_ID_TABLE_DELETE_CELLS)
 #endif
 		EndSubMenu()
@@ -372,11 +367,14 @@ BeginLayout(Main,0)
 		MenuItem(AP_MENU_ID_TABLE_MERGE_CELLS)
 		MenuItem(AP_MENU_ID_TABLE_SPLIT_CELLS)
 #if 0
-// Not for 2.2
+// Not for 2.4
 		MenuItem(AP_MENU_ID_TABLE_SPLIT_TABLE)
 #endif
 		MenuItem(AP_MENU_ID_TABLE_FORMAT)
-		MenuItem(AP_MENU_ID_TABLE_TEXTTOTABLE)
+                BeginSubMenu(AP_MENU_ID_TABLE_TEXTTOTABLE)
+		   MenuItem(AP_MENU_ID_TABLE_TEXTTOTABLE_ALL)
+		   MenuItem(AP_MENU_ID_TABLE_TEXTTOTABLE_NOSPACES)
+                EndSubMenu()
 #if DEBUG
 	    BeginSubMenu(AP_MENU_ID_TABLE_SORT)
 	       MenuItem(AP_MENU_ID_TABLE_SORTROWSASCEND)
@@ -384,15 +382,24 @@ BeginLayout(Main,0)
 	       MenuItem(AP_MENU_ID_TABLE_SORTCOLSASCEND)
 	       MenuItem(AP_MENU_ID_TABLE_SORTCOLSDESCEND)
 	    EndSubMenu()
+#endif
+#if !XP_SIMPLE_MENU
+	    BeginSubMenu(AP_MENU_ID_TABLE_TABLETOTEXT)
+	       MenuItem(AP_MENU_ID_TABLE_TABLETOTEXTCOMMAS)
+	       MenuItem(AP_MENU_ID_TABLE_TABLETOTEXTTABS)
+	       MenuItem(AP_MENU_ID_TABLE_TABLETOTEXTCOMMASTABS)
+	    EndSubMenu()
 #endif
 		MenuItem(AP_MENU_ID_TABLE_AUTOFIT)
-
-#if 0
-// Not for 2.2
-		MenuItem(AP_MENU_ID_TABLE_HEADING_ROWS_REPEAT)
+#if DEBUG
+	    BeginSubMenu(AP_MENU_ID_TABLE_HEADING_ROWS_REPEAT)
+                MenuItem(AP_MENU_ID_TABLE_HEADING_ROWS_REPEAT_THIS)
+                MenuItem(AP_MENU_ID_TABLE_HEADING_ROWS_REPEAT_REMOVE)
+	    EndSubMenu()
 #endif
 	EndSubMenu()
 
+#if !XP_SIMPLE_MENU
 	BeginSubMenu(AP_MENU_ID_WINDOW)
 		MenuItem(AP_MENU_ID_WINDOW_NEW)
 		Separator()
@@ -415,20 +422,16 @@ BeginLayout(Main,0)
 		MenuItem(AP_MENU_ID_HELP_CHECKVER)
 		MenuItem(AP_MENU_ID_HELP_REPORT_BUG)
 		Separator()
-		MenuItem(AP_MENU_ID_HELP_ABOUTOS)
-		MenuItem(AP_MENU_ID_HELP_ABOUT_GNU)
 #ifdef HAVE_GNOME
 		MenuItem(AP_MENU_ID_HELP_ABOUT_GNOMEOFFICE)
 #endif
-		Separator()
 #ifndef HAVE_GNOME
 		MenuItem(AP_MENU_ID_HELP_CREDITS)
-#ifndef XP_MAC_TARGET_MACOSX
-		// On MacOS X don't put a separator as the "About" menu item is moved away at run time
-		Separator()
-#endif
 #endif
 		MenuItem(AP_MENU_ID_HELP_ABOUT)
 	EndSubMenu()
+#endif
 
 EndLayout()
+	
+#endif
