@@ -1,3 +1,5 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiWord
  * Copyright (C) 2001 AbiSource, Inc.
  * 
@@ -17,78 +19,19 @@
  * 02111-1307, USA.
  */
 
-#include "ut_string.h"
 #include "ie_exp_AWT.h"
+#include "ut_string.h"
+#include "ut_assert.h"
 
 /*****************************************************************/
 /*****************************************************************/
 
-#ifdef ENABLE_PLUGINS
-
-// completely generic code to allow this to be a plugin
-
-#include "xap_Module.h"
-
-ABI_PLUGIN_DECLARE("AWT")
-
-// we use a reference-counted sniffer
-static IE_Exp_AWT_Sniffer * m_sniffer = 0;
-
-ABI_FAR_CALL
-int abi_plugin_register (XAP_ModuleInfo * mi)
+IE_Exp_AWT_Sniffer::IE_Exp_AWT_Sniffer ()
+	: IE_ExpSniffer(IE_IMPEXPNAME_AWML11AWT)
 {
-
-	if (!m_sniffer)
-	{
-		m_sniffer = new IE_Exp_AWT_Sniffer ();
-	}
-	else
-	{
-		m_sniffer->ref();
-	}
-
-	mi->name = "AbiWord Template Exporter";
-	mi->desc = "Export AbiWord Documents as Templates";
-	mi->version = ABI_VERSION_STRING;
-	mi->author = "Abi the Ant";
-	mi->usage = "No Usage";
-
-	IE_Exp::registerExporter (m_sniffer);
-	return 1;
+	// 
 }
 
-ABI_FAR_CALL
-int abi_plugin_unregister (XAP_ModuleInfo * mi)
-{
-	mi->name = 0;
-	mi->desc = 0;
-	mi->version = 0;
-	mi->author = 0;
-	mi->usage = 0;
-
-	UT_ASSERT (m_sniffer);
-
-	IE_Exp::unregisterExporter (m_sniffer);
-	if (!m_sniffer->unref())
-	{
-		m_sniffer = 0;
-	}
-
-	return 1;
-}
-
-ABI_FAR_CALL
-int abi_plugin_supports_version (UT_uint32 major, UT_uint32 minor, 
-								 UT_uint32 release)
-{
-	return isCurrentAbiVersion (major, minor, release) ? 1 : 0;
-}
-
-#endif
-
-/*****************************************************************/
-/*****************************************************************/
-
 bool IE_Exp_AWT_Sniffer::recognizeSuffix(const char * szSuffix)
 {
 	return (!UT_stricmp(szSuffix,".awt"));
