@@ -27,17 +27,19 @@
 
 // The importer/reader for GZipped AbiWord file format version 1.
 
-class IE_Imp_GZipAbiWord_Sniffer : public IE_ImpSniffer
+class ABI_EXPORT IE_Imp_GZipAbiWord_Sniffer : public IE_ImpSniffer
 {
 	friend class IE_Imp;
 
 public:
-	IE_Imp_GZipAbiWord_Sniffer() {}
+	IE_Imp_GZipAbiWord_Sniffer();
 	virtual ~IE_Imp_GZipAbiWord_Sniffer() {}
 
-	virtual bool recognizeContents (const char * szBuf, 
+	UT_Confidence_t supportsMIME (const char * szMIME);
+
+	virtual UT_Confidence_t recognizeContents (const char * szBuf, 
 									UT_uint32 iNumbytes);
-	virtual bool recognizeSuffix (const char * szSuffix);
+	virtual UT_Confidence_t recognizeSuffix (const char * szSuffix);
 	virtual bool getDlgLabels (const char ** szDesc,
 							   const char ** szSuffixList,
 							   IEFileType * ft);
@@ -46,19 +48,20 @@ public:
 
 };
 
-class IE_Imp_GZipAbiWord : public IE_Imp_AbiWord_1
+class ABI_EXPORT IE_Imp_GZipAbiWord : public IE_Imp_AbiWord_1, public UT_XML::Reader
 {
 public:
 	IE_Imp_GZipAbiWord(PD_Document * pDocument);
 	~IE_Imp_GZipAbiWord();
 
-protected:
-    virtual bool			_openFile(const char * szFilename);
-    virtual UT_uint32			_readBytes(char * buf, UT_uint32 length);
-    virtual void			_closeFile(void);
+	/* Implementation of UT_XML::Reader
+	 */
+	virtual bool      openFile (const char * szFilename);
+	virtual UT_uint32 readBytes (char * buffer, UT_uint32 length);
+	virtual void      closeFile (void);
 
- private:
-    gzFile m_gzfp;
+private:
+	gzFile m_gzfp;
 };
 
 #endif /* IE_IMP_GZIPABIWORD_H */
