@@ -10,11 +10,16 @@ _rtf_keyword rtfKeywords[] = {
 	{"ab", false, false, NO_CONTEXT, RTF_KW_ab }, 
 	{"abibotline", false, false, NO_CONTEXT, RTF_KW_abibotline }, 
 	{"abicellprops", false, false, NO_CONTEXT, RTF_KW_abicellprops }, 
+	{"abiembed", false, false, NO_CONTEXT, RTF_KW_abiembed }, 
 	{"abiendcell", false, false, NO_CONTEXT, RTF_KW_abiendcell }, 
 	{"abiendtable", false, false, NO_CONTEXT, RTF_KW_abiendtable }, 
 	{"abifieldD", false, false, NO_CONTEXT, RTF_KW_abifieldD }, 
 	{"abilist", false, false, NO_CONTEXT, RTF_KW_abilist }, 
+	{"abiltr", false, false, NO_CONTEXT, RTF_KW_abiltr }, 
+	{"abimathml",false,false,NO_CONTEXT,RTF_KW_abimathml },
 	{"abinodiroverride", false, false, NO_CONTEXT, RTF_KW_abinodiroverride }, 
+	{"abirtl", false, false, NO_CONTEXT, RTF_KW_abirtl }, 
+	{"abirevision", false, false, NO_CONTEXT, RTF_KW_abirevision }, 
 	{"abitableprops", false, false, NO_CONTEXT, RTF_KW_abitableprops }, 
 	{"abitopline", false, false, NO_CONTEXT, RTF_KW_abitopline }, 
 	{"absh", false, false, NO_CONTEXT, RTF_KW_absh }, 
@@ -151,6 +156,7 @@ _rtf_keyword rtfKeywords[] = {
 	{"brdrinset", false, false, NO_CONTEXT, RTF_KW_brdrinset }, /* 2000 */ 
 	{"brdrl", false, false, NO_CONTEXT, RTF_KW_brdrl }, 
 	{"brdrnil", false, false, NO_CONTEXT, RTF_KW_brdrnil }, /* 2002 */ 
+	{"brdrnone", false, false, NO_CONTEXT, RTF_KW_brdrnone }, /* 2002 */ 
 	{"brdroutset", false, false, NO_CONTEXT, RTF_KW_brdroutset }, /* 2000 */ 
 	{"brdrr", false, false, NO_CONTEXT, RTF_KW_brdrr }, 
 	{"brdrs", false, false, NO_CONTEXT, RTF_KW_brdrs }, 
@@ -510,6 +516,7 @@ _rtf_keyword rtfKeywords[] = {
 	{"fs", false, false, NO_CONTEXT, RTF_KW_fs }, 
 	{"fscript", false, false, NO_CONTEXT, RTF_KW_fscript }, 
 	{"fswiss", false, false, NO_CONTEXT, RTF_KW_fswiss }, 
+	{"ftech", false, false, NO_CONTEXT, RTF_KW_ftech },
 	{"ftnalt", false, false, NO_CONTEXT, RTF_KW_ftnalt }, 
 	{"ftnbj", false, false, NO_CONTEXT, RTF_KW_ftnbj }, 
 	{"ftncn", false, false, NO_CONTEXT, RTF_KW_ftncn }, 
