@@ -493,7 +493,7 @@ bool s_XSL_FO_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 			      const char* dataid = strdup(static_cast<const char*>(szValue));
 			      const XML_Char * szWidth = 0;
 			      const XML_Char * szHeight = 0;
-			      m_pie->write(UT_String_sprintf("<fo:external-graphic src=\"url('%s-%d.png')\"/", m_pie->getFileName(), m_iImgCnt++));
+			      m_pie->write(UT_String_sprintf("<fo:external-graphic src=\"url('%s-%d.png')\"", m_pie->getFileName(), m_iImgCnt++));
 
 			      const char * old_locale = setlocale(LC_NUMERIC, "C");
 			      
@@ -698,8 +698,8 @@ bool s_XSL_FO_Listener::populateStrux(PL_StruxDocHandle sdh,
 	case PTX_SectionFootnote:
 	case PTX_EndEndnote:
 	default:
-		UT_ASSERT_NOT_REACHED();
-		return false;
+		UT_ASSERT(UT_TODO);
+		return true;
 	}
 }
 
@@ -824,9 +824,9 @@ void s_XSL_FO_Listener::_handleDataItems()
 	  FILE *fp;
 	  UT_String fname;
 	  
-	  if (!UT_strcmp(szMimeType, "image/svg-xml"))
+	  if (!UT_strcmp(szMimeType, "image/svg+xml"))
 	    UT_String_sprintf(fname, "%s-%d.svg", m_pie->getFileName(), k);
-	  if (!UT_strcmp(szMimeType, "text/mathml"))
+	  if (!UT_strcmp(szMimeType, "application/mathml+xml"))
 	    UT_String_sprintf(fname, "%s-%d.mathml", m_pie->getFileName(), k);
 	  else // PNG Image
 	    UT_String_sprintf(fname, "%s-%d.png", m_pie->getFileName(), k);
